from django.contrib.postgres.indexes import BTreeIndex
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from parler.models import TranslatableModel, TranslatedFields

from core.models import SortableModel, TimeStampMixinModel, UUIDModel
from region.managers import RegionManager


class Region(TranslatableModel, TimeStampMixinModel, SortableModel, UUIDModel):
    alpha = models.CharField(
        _("Region Code"), max_length=10, primary_key=True, unique=True
    )
    country = models.ForeignKey(
        "country.Country", related_name="regions", on_delete=models.CASCADE
    )
    translations = TranslatedFields(
        name=models.CharField(_("Name"), max_length=100, blank=True, null=True)
    )

    objects: RegionManager = RegionManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Region")
        verbose_name_plural = _("Regions")
        ordering = ["sort_order"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            *SortableModel.Meta.indexes,
            BTreeIndex(fields=["alpha"], name="region_alpha_ix"),
            BTreeIndex(fields=["country"], name="region_country_ix"),
        ]

    def __str__(self):
        country_name = self.country.safe_translation_getter(
            "name", any_language=True
        )
        region_name = self.safe_translation_getter("name", any_language=True)
        return f"{region_name}, {country_name}"

    def get_ordering_queryset(self):
        return Region.objects.all()



from django_filters import rest_framework as filters

from core.filters.camel_case_filters import CamelCaseTimeStampFilterSet
from core.filters.core import (
    UUIDFilterMixin,
    SoftDeleteFilterMixin,
    MetaDataFilterMixin,
)
from product.models.category import ProductCategory
from product.models.product import Product


class ProductFilter(
    UUIDFilterMixin,
    SoftDeleteFilterMixin,
    MetaDataFilterMixin,
    CamelCaseTimeStampFilterSet,
):
    id = filters.NumberFilter(field_name="id")
    sku = filters.CharFilter(field_name="sku", lookup_expr="icontains")
    active = filters.BooleanFilter(field_name="active")

    min_price = filters.NumberFilter(
        field_name="price",
        lookup_expr="gte",
        help_text="Minimum price",
    )
    max_price = filters.NumberFilter(
        field_name="price",
        lookup_expr="lte",
        help_text="Maximum price",
    )
    min_final_price = filters.NumberFilter(
        method="filter_min_final_price",
        help_text="Minimum final price (after VAT and discount)",
    )
    max_final_price = filters.NumberFilter(
        method="filter_max_final_price",
        help_text="Maximum final price (after VAT and discount)",
    )

    min_stock = filters.NumberFilter(
        field_name="stock",
        lookup_expr="gte",
        help_text="Minimum stock quantity",
    )
    max_stock = filters.NumberFilter(
        field_name="stock",
        lookup_expr="lte",
        help_text="Maximum stock quantity",
    )
    in_stock = filters.BooleanFilter(
        method="filter_in_stock",
        help_text="Filter products that are in stock (stock > 0)",
    )

    min_discount_percent = filters.NumberFilter(
        field_name="discount_percent",
        lookup_expr="gte",
        help_text="Minimum discount percentage",
    )
    max_discount_percent = filters.NumberFilter(
        field_name="discount_percent",
        lookup_expr="lte",
        help_text="Maximum discount percentage",
    )
    min_discount = filters.NumberFilter(
        method="filter_min_discount",
        help_text="Minimum discount value amount",
    )
    max_discount = filters.NumberFilter(
        method="filter_max_discount",
        help_text="Maximum discount value amount",
    )
    has_discount = filters.BooleanFilter(
        method="filter_has_discount",
        help_text="Filter products that have a discount",
    )

    category = filters.CharFilter(
        field_name="category__id",
        method="filter_category",
        help_text="Filter by category ID (supports multiple IDs separated by underscore)",
    )
    category_id = filters.NumberFilter(
        field_name="category__id",
        help_text="Filter by exact category ID",
    )

    min_view_count = filters.NumberFilter(
        field_name="view_count",
        lookup_expr="gte",
        help_text="Minimum view count",
    )
    max_view_count = filters.NumberFilter(
        field_name="view_count",
        lookup_expr="lte",
        help_text="Maximum view count",
    )

    min_review_average = filters.NumberFilter(
        method="filter_min_review_average",
        help_text="Minimum review average rating",
    )
    max_review_average = filters.NumberFilter(
        method="filter_max_review_average",
        help_text="Maximum review average rating",
    )

    min_likes = filters.NumberFilter(
        method="filter_min_likes",
        help_text="Minimum likes count",
    )
    max_likes = filters.NumberFilter(
        method="filter_max_likes",
        help_text="Maximum likes count",
    )

    min_weight = filters.NumberFilter(
        field_name="weight",
        lookup_expr="gte",
        help_text="Minimum weight",
    )
    max_weight = filters.NumberFilter(
        field_name="weight",
        lookup_expr="lte",
        help_text="Maximum weight",
    )

    class Meta:
        model = Product
        fields = {
            "id": ["exact"],
            "sku": ["exact", "icontains"],
            "active": ["exact"],
            "price": ["exact", "gte", "lte"],
            "stock": ["exact", "gte", "lte"],
            "discount_percent": ["exact", "gte", "lte"],
            "view_count": ["exact", "gte", "lte"],
            "weight": ["exact", "gte", "lte"],
            "category": ["exact"],
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "deleted_at": ["gte", "lte", "date"],
            "is_deleted": ["exact"],
            "uuid": ["exact"],
        }

    def filter_category(self, queryset, name, value):
        """Filter by category including descendant categories"""
        category_ids = value.split("_")
        all_relevant_category_ids = []

        for category_id in category_ids:
            try:
                category = ProductCategory.objects.get(id=category_id)
                descendant_ids = category.get_descendants(
                    include_self=True
                ).values_list("id", flat=True)
                all_relevant_category_ids.extend(descendant_ids)
            except ProductCategory.DoesNotExist:
                pass

        return queryset.filter(category__id__in=all_relevant_category_ids)

    def filter_in_stock(self, queryset, name, value):
        """Filter products that are in stock"""
        if value:
            return queryset.filter(stock__gt=0)
        return queryset.filter(stock=0)

    def filter_has_discount(self, queryset, name, value):
        """Filter products that have a discount"""
        if value:
            return queryset.filter(discount_percent__gt=0)
        return queryset.filter(discount_percent=0)

    def filter_min_final_price(self, queryset, name, value):
        """Filter products with minimum final price"""
        if value is not None:
            return queryset.with_final_price_annotation().filter(
                final_price_annotation__gte=value
            )
        return queryset

    def filter_max_final_price(self, queryset, name, value):
        """Filter products with maximum final price"""
        if value is not None:
            return queryset.with_final_price_annotation().filter(
                final_price_annotation__lte=value
            )
        return queryset

    def filter_min_discount(self, queryset, name, value):
        """Filter products with minimum discount value"""
        if value is not None:
            return queryset.with_discount_value_annotation().filter(
                discount_value_annotation__gte=value
            )
        return queryset

    def filter_max_discount(self, queryset, name, value):
        """Filter products with maximum discount value"""
        if value is not None:
            return queryset.with_discount_value_annotation().filter(
                discount_value_annotation__lte=value
            )
        return queryset

    def filter_min_review_average(self, queryset, name, value):
        """Filter products with minimum review average"""
        if value is not None:
            return queryset.with_review_average_annotation().filter(
                review_average_annotation__gte=value
            )
        return queryset

    def filter_max_review_average(self, queryset, name, value):
        """Filter products with maximum review average"""
        if value is not None:
            return queryset.with_review_average_annotation().filter(
                review_average_annotation__lte=value
            )
        return queryset

    def filter_min_likes(self, queryset, name, value):
        """Filter products with minimum likes count"""
        if value is not None:
            return queryset.with_likes_count_annotation().filter(
                likes_count_annotation__gte=value
            )
        return queryset

    def filter_max_likes(self, queryset, name, value):
        """Filter products with maximum likes count"""
        if value is not None:
            return queryset.with_likes_count_annotation().filter(
                likes_count_annotation__lte=value
            )
        return queryset



import io
import os
from unittest.mock import patch

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase, override_settings

from core.forms.image import ImageAndSvgField


class ImageAndSvgFieldTest(TestCase):
    def setUp(self):
        self.field = ImageAndSvgField()
        self.field_with_size_limit = ImageAndSvgField(max_file_size=1024)

    def create_test_file(self, content, filename, content_type="text/plain"):
        if isinstance(content, str):
            content = content.encode("utf-8")
        return SimpleUploadedFile(filename, content, content_type=content_type)

    def get_static_file(self, filename):
        static_path = os.path.join(
            settings.BASE_DIR, "static", "images", "test", filename
        )
        if os.path.exists(static_path):
            with open(static_path, "rb") as f:
                content = f.read()
            return SimpleUploadedFile(filename, content)
        return None

    def test_valid_svg_file(self):
        valid_svg = """<?xml version="1.0" encoding="UTF-8"?>
        <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
            <circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red" />
        </svg>"""
        svg_file = self.create_test_file(valid_svg, "test.svg", "image/svg+xml")

        result = self.field.to_python(svg_file)
        self.assertIsNotNone(result)
        self.assertEqual(result.content_type, "image/svg+xml")

    def test_svg_without_namespace(self):
        simple_svg = """<svg width="100" height="100">
            <rect width="50" height="50" fill="blue" />
        </svg>"""
        svg_file = self.create_test_file(
            simple_svg, "simple.svg", "image/svg+xml"
        )

        result = self.field.to_python(svg_file)
        self.assertIsNotNone(result)

    def test_svg_with_script_element_rejected(self):
        malicious_svg = """<?xml version="1.0" encoding="UTF-8"?>
        <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
            <script>alert('XSS')</script>
            <circle cx="50" cy="50" r="40" fill="red" />
        </svg>"""
        svg_file = self.create_test_file(
            malicious_svg, "malicious.svg", "image/svg+xml"
        )

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(svg_file)
        self.assertEqual(cm.exception.code, "svg_script_not_allowed")

    def test_svg_with_event_handlers_rejected(self):
        malicious_svg = """<?xml version="1.0" encoding="UTF-8"?>
        <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
            <circle cx="50" cy="50" r="40" onclick="alert('XSS')" fill="red" />
        </svg>"""
        svg_file = self.create_test_file(
            malicious_svg, "onclick.svg", "image/svg+xml"
        )

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(svg_file)
        self.assertEqual(cm.exception.code, "svg_event_handler_not_allowed")

    def test_svg_with_multiple_event_handlers_rejected(self):
        event_handlers = [
            "onclick",
            "onload",
            "onmouseover",
            "onfocus",
            "onblur",
        ]

        for handler in event_handlers:
            with self.subTest(handler=handler):
                malicious_svg = f"""<?xml version="1.0" encoding="UTF-8"?>
                <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
                    <rect {handler}="alert('XSS')" width="50" height="50" fill="blue" />
                </svg>"""
                svg_file = self.create_test_file(
                    malicious_svg, f"{handler}.svg", "image/svg+xml"
                )

                with self.assertRaises(ValidationError) as cm:
                    self.field.to_python(svg_file)
                self.assertEqual(
                    cm.exception.code, "svg_event_handler_not_allowed"
                )

    def test_invalid_xml_rejected(self):
        invalid_xml = """<svg><unclosed tag</svg>"""
        svg_file = self.create_test_file(
            invalid_xml, "invalid.svg", "image/svg+xml"
        )

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(svg_file)
        self.assertEqual(cm.exception.code, "invalid_image")

    def test_non_svg_xml_rejected(self):
        xml_content = """<?xml version="1.0"?><root><item>test</item></root>"""
        xml_file = self.create_test_file(xml_content, "test.xml", "text/xml")

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(xml_file)
        self.assertEqual(cm.exception.code, "invalid_image")

    def test_text_file_rejected(self):
        text_file = self.create_test_file(
            "This is just text", "test.txt", "text/plain"
        )

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(text_file)
        self.assertEqual(cm.exception.code, "invalid_image")

    def test_none_value(self):
        result = self.field.to_python(None)
        self.assertIsNone(result)

    def test_file_size_validation(self):
        large_content = "x" * 2048
        large_file = self.create_test_file(large_content, "large.txt")
        large_file.size = 2048

        with self.assertRaises(ValidationError) as cm:
            self.field_with_size_limit.to_python(large_file)
        self.assertEqual(cm.exception.code, "file_too_large")
        self.assertIn("1.0KB", str(cm.exception))

    def test_file_size_validation_svg(self):
        large_svg = (
            """<?xml version="1.0" encoding="UTF-8"?>
                <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">"""
            + "<!-- "
            + "x" * 2000
            + " -->"
            + """
                    <circle cx="50" cy="50" r="40" fill="red" />
                </svg>"""
        )
        large_svg_file = self.create_test_file(
            large_svg, "large.svg", "image/svg+xml"
        )
        large_svg_file.size = len(large_svg.encode("utf-8"))

        with self.assertRaises(ValidationError) as cm:
            self.field_with_size_limit.to_python(large_svg_file)
        self.assertEqual(cm.exception.code, "file_too_large")

    def test_file_extension_validation(self):
        exe_file = self.create_test_file("fake exe content", "malware.exe")

        with self.assertRaises(ValidationError):
            self.field.to_python(exe_file)

    @override_settings(FILE_UPLOAD_MAX_MEMORY_SIZE=1048576)
    def test_default_max_file_size_from_settings(self):
        field = ImageAndSvgField()
        self.assertEqual(field.max_file_size, 1048576)

    def test_default_max_file_size_fallback(self):
        with patch.object(settings, "FILE_UPLOAD_MAX_MEMORY_SIZE", None):
            field = ImageAndSvgField()
            self.assertEqual(field.max_file_size, 2621440)

    def test_custom_max_file_size(self):
        custom_size = 5242880
        field = ImageAndSvgField(max_file_size=custom_size)
        self.assertEqual(field.max_file_size, custom_size)

    def test_file_seeking_behavior(self):
        valid_svg = """<?xml version="1.0" encoding="UTF-8"?>
        <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
            <circle cx="50" cy="50" r="40" fill="red" />
        </svg>"""

        file_content = io.BytesIO(valid_svg.encode("utf-8"))
        file_content.seek(50)
        file_content.tell()

        svg_file = SimpleUploadedFile(
            "test.svg", file_content.getvalue(), "image/svg+xml"
        )
        svg_file.file = file_content

        result = self.field.to_python(svg_file)
        self.assertIsNotNone(result)

    def test_static_image_files(self):
        test_files = [
            "test_image.jpg",
            "test_image.png",
            "test_image.gif",
            "test_svg.svg",
        ]

        for filename in test_files:
            with self.subTest(filename=filename):
                static_file = self.get_static_file(filename)
                if static_file:
                    try:
                        result = self.field.to_python(static_file)
                        self.assertIsNotNone(result)
                    except ValidationError:
                        pass

    def test_is_svg_static_method(self):
        self.assertFalse(ImageAndSvgField.is_svg(None))

        valid_svg = """<svg><rect width="50" height="50"/></svg>"""
        svg_file = self.create_test_file(valid_svg, "test.svg")
        self.assertTrue(ImageAndSvgField.is_svg(svg_file))

        invalid_file = self.create_test_file("not xml", "test.txt")
        self.assertFalse(ImageAndSvgField.is_svg(invalid_file))

    def test_svg_case_insensitive_security_check(self):
        malicious_svg = """<svg><SCRIPT>alert('test')</SCRIPT></svg>"""
        svg_file = self.create_test_file(malicious_svg, "upper.svg")

        with self.assertRaises(ValidationError) as cm:
            self.field.to_python(svg_file)
        self.assertEqual(cm.exception.code, "svg_script_not_allowed")



from django.db import IntegrityError
from django.test import TestCase

from product.enum.category import CategoryImageTypeEnum
from product.factories.category import ProductCategoryFactory
from product.factories.category_image import ProductCategoryImageFactory
from product.models.category_image import ProductCategoryImage


class ProductCategoryImageModelTestCase(TestCase):
    def setUp(self):
        self.category = ProductCategoryFactory()

    def test_create_category_image(self):
        image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.MAIN
        )

        self.assertEqual(image.category, self.category)
        self.assertEqual(image.image_type, CategoryImageTypeEnum.MAIN)
        self.assertTrue(image.active)
        self.assertIsNotNone(image.image)

    def test_unique_image_type_per_category(self):
        ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.MAIN
        )

        with self.assertRaises(IntegrityError):
            ProductCategoryImageFactory(
                category=self.category, image_type=CategoryImageTypeEnum.MAIN
            )

    def test_multiple_image_types_same_category(self):
        main_image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.MAIN
        )
        banner_image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.BANNER
        )

        self.assertEqual(main_image.category, self.category)
        self.assertEqual(banner_image.category, self.category)
        self.assertNotEqual(main_image.image_type, banner_image.image_type)

    def test_str_representation(self):
        image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.BANNER
        )

        category_name = self.category.safe_translation_getter(
            "name", any_language=True
        )
        expected_str = f"{category_name} - Banner Image"
        self.assertEqual(str(image), expected_str)

    def test_class_methods(self):
        main_image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.MAIN
        )
        banner_image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.BANNER
        )

        retrieved_main = ProductCategoryImage.get_main_image(self.category)
        retrieved_banner = ProductCategoryImage.get_banner_image(self.category)
        retrieved_by_type = ProductCategoryImage.get_image_by_type(
            self.category, CategoryImageTypeEnum.MAIN
        )

        self.assertEqual(retrieved_main, main_image)
        self.assertEqual(retrieved_banner, banner_image)
        self.assertEqual(retrieved_by_type, main_image)

    def test_image_properties(self):
        image = ProductCategoryImageFactory(
            category=self.category, image_type=CategoryImageTypeEnum.ICON
        )

        self.assertIsNotNone(image.image_path)
        self.assertIsNotNone(image.image_url)
        self.assertIn("category", image.image_path)

    def test_manager_methods(self):
        active_image = ProductCategoryImageFactory(
            category=self.category,
            active=True,
            image_type=CategoryImageTypeEnum.GALLERY,
        )
        inactive_image = ProductCategoryImageFactory(
            category=self.category,
            active=False,
            image_type=CategoryImageTypeEnum.THUMBNAIL,
        )

        active_images = ProductCategoryImage.objects.active()
        category_images = ProductCategoryImage.objects.by_category(
            self.category
        )

        self.assertIn(active_image, active_images)
        self.assertNotIn(inactive_image, active_images)
        self.assertIn(active_image, category_images)
        self.assertIn(inactive_image, category_images)



# Generated by Django 5.0.6 on 2024-07-02 13:39
from django.db import migrations

import core.fields.image


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0013_alter_producttranslation_description_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="productimage",
            name="image",
            field=core.fields.image.ImageAndSvgField(
                upload_to="uploads/products/", verbose_name="Image"
            ),
        ),
    ]



from django.test import TestCase
from rest_framework.test import APIRequestFactory

from product.serializers.favourite import (
    ProductDetailResponseSerializer,
    ProductFavouriteByProductsResponseSerializer,
    ProductFavouriteDetailSerializer,
    ProductFavouriteSerializer,
    ProductFavouriteWriteSerializer,
)
from product.views.favourite import ProductFavouriteViewSet


class ProductFavouriteViewSetTestCase(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.viewset = ProductFavouriteViewSet()
        self.viewset.action = "list"

    def test_list_action_serializer(self):
        self.viewset.action = "list"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteSerializer)

    def test_retrieve_action_serializer(self):
        self.viewset.action = "retrieve"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteDetailSerializer)

    def test_create_action_serializer(self):
        self.viewset.action = "create"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteWriteSerializer)

    def test_partial_update_action_serializer(self):
        self.viewset.action = "partial_update"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteWriteSerializer)

    def test_custom_action_serializers(self):
        self.viewset.action = "product"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductDetailResponseSerializer)

        self.viewset.action = "favourites_by_products"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(
            serializer_class, ProductFavouriteByProductsResponseSerializer
        )

    def test_queryset_optimization(self):
        queryset = self.viewset.get_queryset()

        self.assertIn("user", str(queryset.query))
        self.assertIn("product", str(queryset.query))

    def test_ordering_fields_configuration(self):
        expected_fields = [
            "id",
            "user_id",
            "product_id",
            "created_at",
            "updated_at",
        ]
        self.assertEqual(self.viewset.ordering_fields, expected_fields)

    def test_search_fields_configuration(self):
        expected_fields = [
            "user__username",
            "product__translations__name",
        ]
        self.assertEqual(self.viewset.search_fields, expected_fields)

    def test_default_ordering(self):
        expected_ordering = ["-created_at"]
        self.assertEqual(self.viewset.ordering, expected_ordering)

    def test_get_serializer_for_schema(self):
        self.viewset.action = "list"
        serializer = self.viewset.get_serializer_class()
        self.assertEqual(serializer, ProductFavouriteSerializer)

        self.viewset.action = "create"
        serializer = self.viewset.get_serializer_class()
        self.assertEqual(serializer, ProductFavouriteWriteSerializer)

    def test_request_serializer_class(self):
        write_actions = ["create", "update", "partial_update"]
        for action in write_actions:
            self.viewset.action = action
            serializer_class = self.viewset.get_serializer_class()
            self.assertEqual(serializer_class, ProductFavouriteWriteSerializer)

    def test_response_serializer_class(self):
        self.viewset.action = "list"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteSerializer)

        self.viewset.action = "retrieve"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductFavouriteDetailSerializer)

    def test_response_serializers_configuration(self):
        response_serializers = {
            "list": ProductFavouriteSerializer,
            "retrieve": ProductFavouriteDetailSerializer,
            "create": ProductFavouriteDetailSerializer,
            "update": ProductFavouriteDetailSerializer,
            "partial_update": ProductFavouriteDetailSerializer,
            "product": ProductDetailResponseSerializer,
            "favourites_by_products": ProductFavouriteByProductsResponseSerializer,
        }

        for action, expected_serializer in response_serializers.items():
            self.viewset.action = action
            if action in ["create", "update", "partial_update"]:
                actual_serializer = self.viewset.response_serializers.get(
                    "default", ProductFavouriteDetailSerializer
                )
            else:
                actual_serializer = self.viewset.get_serializer_class()

            if action not in ["create", "update", "partial_update"]:
                self.assertEqual(
                    actual_serializer,
                    expected_serializer,
                    f"Action '{action}' should use {expected_serializer.__name__}",
                )



import logging
from typing import Any
from extra_settings.models import Setting
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.tokens import default_token_generator
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.encoding import force_bytes
from django.utils.html import strip_tags
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import gettext as _

from user.models.subscription import SubscriptionTopic, UserSubscription

logger = logging.getLogger(__name__)
User = get_user_model()


def send_subscription_confirmation(
    subscription: UserSubscription, user: User
) -> bool:
    if check_subscription_before_send(
        user=user, topic_slug=subscription.topic.slug
    ):
        logger.warning(
            f"Attempted to send confirmation for already active subscription {subscription.id}"
        )
        return False

    if subscription.status != UserSubscription.SubscriptionStatus.PENDING:
        logger.warning(
            f"Attempted to send confirmation for non-pending subscription {subscription.id}"
        )
        return False

    if not subscription.confirmation_token:
        logger.error(
            f"No confirmation token for subscription {subscription.id}"
        )
        return False

    try:
        confirmation_url = Setting.get("SUBSCRIPTION_CONFIRMATION_URL")

        context = {
            "user": subscription.user,
            "topic": subscription.topic,
            "confirmation_url": confirmation_url.format(
                token=subscription.confirmation_token
            ),
            "site_name": getattr(settings, "SITE_NAME", "Our Site"),
            "support_email": getattr(
                settings, "SUPPORT_EMAIL", "support@example.com"
            ),
        }

        subject = _("Confirm your subscription to {topic}").format(
            topic=subscription.topic.name
        )

        # @TODO: create email template
        html_message = render_to_string(
            "emails/subscription/confirmation.html", context
        )

        text_message = strip_tags(html_message)

        email = EmailMultiAlternatives(
            subject=subject,
            body=text_message,
            from_email=settings.DEFAULT_FROM_EMAIL,
            to=[subscription.user.email],
        )
        email.attach_alternative(html_message, "text/html")
        email.send()

        logger.info(
            f"Sent confirmation email for subscription {subscription.id}"
        )
        return True

    except Exception as e:
        logger.error(
            f"Failed to send confirmation email for subscription {subscription.id}: {e}"
        )
        return False


def check_subscription_before_send(user: "User", topic_slug: str) -> bool:
    return UserSubscription.objects.filter(
        user=user,
        topic__slug=topic_slug,
        status=UserSubscription.SubscriptionStatus.ACTIVE,
    ).exists()


def generate_unsubscribe_link(user: "User", topic: SubscriptionTopic) -> str:
    token = default_token_generator.make_token(user)
    uid = urlsafe_base64_encode(force_bytes(user.pk))

    base_url = getattr(settings, "SITE_URL", "https://example.com")
    unsubscribe_url = f"{base_url}/unsubscribe/{uid}/{token}/{topic.slug}/"

    return unsubscribe_url


def send_newsletter(
    topic: SubscriptionTopic,
    subject: str,
    template_name: str,
    context: dict[str, Any],
    batch_size: int = 100,
) -> dict[str, int]:
    stats = {"sent": 0, "failed": 0, "skipped": 0}

    active_subscriptions = UserSubscription.objects.filter(
        topic=topic, status=UserSubscription.SubscriptionStatus.ACTIVE
    ).select_related("user")

    for subscription in active_subscriptions.iterator(chunk_size=batch_size):
        user = subscription.user

        if not user.is_active:
            stats["skipped"] += 1
            continue

        user_context = context.copy()
        user_context.update(
            {
                "user": user,
                "topic": topic,
                "subscription": subscription,
                "unsubscribe_url": generate_unsubscribe_link(user, topic),
                "preferences_url": f"{getattr(settings, 'SITE_URL', '')}/account/subscriptions/",
            }
        )

        try:
            html_message = render_to_string(template_name, user_context)
            text_message = strip_tags(html_message)

            email = EmailMultiAlternatives(
                subject=subject,
                body=text_message,
                from_email=settings.DEFAULT_FROM_EMAIL,
                to=[user.email],
            )
            email.attach_alternative(html_message, "text/html")
            email.send()

            stats["sent"] += 1

        except Exception as e:
            logger.error(f"Failed to send newsletter to {user.email}: {e}")
            stats["failed"] += 1

    logger.info(
        f"Newsletter sent for topic {topic.slug}: "
        f"{stats['sent']} sent, {stats['failed']} failed, {stats['skipped']} skipped"
    )

    return stats


def get_user_subscription_summary(user: "User") -> dict[str, Any]:
    subscriptions = UserSubscription.objects.filter(user=user).select_related(
        "topic"
    )

    summary = {
        "total": subscriptions.count(),
        "active": subscriptions.filter(
            status=UserSubscription.SubscriptionStatus.ACTIVE
        ).count(),
        "pending": subscriptions.filter(
            status=UserSubscription.SubscriptionStatus.PENDING
        ).count(),
        "unsubscribed": subscriptions.filter(
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED
        ).count(),
        "by_category": {},
    }

    for subscription in subscriptions:
        category = subscription.topic.category
        if category not in summary["by_category"]:
            summary["by_category"][category] = {
                "total": 0,
                "active": 0,
            }

        summary["by_category"][category]["total"] += 1
        if subscription.status == UserSubscription.SubscriptionStatus.ACTIVE:
            summary["by_category"][category]["active"] += 1

    return summary



from __future__ import annotations

from typing import TYPE_CHECKING

from django.contrib.auth.models import BaseUserManager

from core.generators import UserNameGenerator

if TYPE_CHECKING:
    from user.models import UserAccount


class UserAccountManager(BaseUserManager["UserAccount"]):
    def active(self):
        return self.filter(is_active=True)

    def inactive(self):
        return self.filter(is_active=False)

    def staff(self):
        return self.filter(is_staff=True)

    def create_user(
        self, email: str, password: str, **extra_fields
    ) -> UserAccount:
        if not email:
            raise ValueError("Users must have an email address")
        new_email: str = self.normalize_email(email)
        username = extra_fields.pop(
            "username", None
        ) or UserNameGenerator().generate_username(new_email)
        user = self.model(email=new_email, username=username, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def _create_user(
        self, email: str, password: str, **extra_fields
    ) -> UserAccount:
        if not email:
            raise ValueError("Users must have an email address")
        new_email: str = self.normalize_email(email)
        username = extra_fields.pop(
            "username", None
        ) or UserNameGenerator().generate_username(new_email)
        user = self.model(email=new_email, username=username, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(
        self, email: str, password: str, **extra_fields
    ) -> UserAccount:
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)

        if not extra_fields.get("is_staff"):
            raise ValueError("Superuser must have is_staff=True.")
        if not extra_fields.get("is_superuser"):
            raise ValueError("Superuser must have is_superuser=True.")

        return self._create_user(email, password, **extra_fields)



from django_filters import rest_framework as filters

from core.filters.camel_case_filters import CamelCaseTimeStampFilterSet
from core.filters.core import (
    UUIDFilterMixin,
    SortableFilterMixin,
)
from product.models.category import ProductCategory


class ProductCategoryFilter(
    UUIDFilterMixin,
    SortableFilterMixin,
    CamelCaseTimeStampFilterSet,
):
    id = filters.NumberFilter(field_name="id")
    slug = filters.CharFilter(field_name="slug", lookup_expr="icontains")
    active = filters.BooleanFilter(field_name="active")

    sort_order = filters.NumberFilter(
        field_name="sort_order",
        help_text="Filter by exact sort order",
    )
    sort_order_min = filters.NumberFilter(
        field_name="sort_order",
        lookup_expr="gte",
        help_text="Minimum sort order",
    )
    sort_order_max = filters.NumberFilter(
        field_name="sort_order",
        lookup_expr="lte",
        help_text="Maximum sort order",
    )

    parent = filters.NumberFilter(
        field_name="parent__id",
        help_text="Filter by parent category ID",
    )
    parent_slug = filters.CharFilter(
        field_name="parent__slug",
        help_text="Filter by parent category slug",
    )
    level = filters.NumberFilter(
        field_name="level",
        help_text="Filter by hierarchy level (0 = root)",
    )
    min_level = filters.NumberFilter(
        field_name="level",
        lookup_expr="gte",
        help_text="Minimum hierarchy level",
    )
    max_level = filters.NumberFilter(
        field_name="level",
        lookup_expr="lte",
        help_text="Maximum hierarchy level",
    )

    is_root = filters.BooleanFilter(
        method="filter_is_root",
        help_text="Filter root categories (no parent)",
    )
    is_leaf = filters.BooleanFilter(
        method="filter_is_leaf",
        help_text="Filter leaf categories (no children)",
    )
    has_children = filters.BooleanFilter(
        method="filter_has_children",
        help_text="Filter categories that have children",
    )

    min_product_count = filters.NumberFilter(
        method="filter_min_product_count",
        help_text="Minimum number of products (recursive)",
    )
    max_product_count = filters.NumberFilter(
        method="filter_max_product_count",
        help_text="Maximum number of products (recursive)",
    )
    has_products = filters.BooleanFilter(
        method="filter_has_products",
        help_text="Filter categories that have products (recursive)",
    )

    ancestor_of = filters.NumberFilter(
        method="filter_ancestor_of",
        help_text="Filter ancestors of specified category ID",
    )
    descendant_of = filters.NumberFilter(
        method="filter_descendant_of",
        help_text="Filter descendants of specified category ID",
    )
    sibling_of = filters.NumberFilter(
        method="filter_sibling_of",
        help_text="Filter siblings of specified category ID",
    )

    class Meta:
        model = ProductCategory
        fields = {
            "id": ["exact"],
            "slug": ["exact", "icontains"],
            "active": ["exact"],
            "parent": ["exact"],
            "level": ["exact", "gte", "lte"],
            "sort_order": ["exact"],
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "uuid": ["exact"],
        }

    def filter_is_root(self, queryset, name, value):
        """Filter root categories (no parent)"""
        if value:
            return queryset.filter(parent__isnull=True)
        return queryset.filter(parent__isnull=False)

    def filter_is_leaf(self, queryset, name, value):
        """Filter leaf categories (no children)"""
        if value:
            return queryset.filter(children__isnull=True)
        return queryset.filter(children__isnull=False)

    def filter_has_children(self, queryset, name, value):
        """Filter categories that have children"""
        if value:
            return queryset.filter(children__isnull=False).distinct()
        return queryset.filter(children__isnull=True)

    def filter_min_product_count(self, queryset, name, value):
        """Filter categories with minimum product count (recursive)"""
        if not value or value <= 0:
            return queryset

        from product.models.product import Product

        category_ids = []
        for category in queryset:
            descendants = category.get_descendants(include_self=True)
            product_count = Product.objects.filter(
                category__in=descendants
            ).count()
            if product_count >= value:
                category_ids.append(category.id)

        return queryset.filter(id__in=category_ids)

    def filter_max_product_count(self, queryset, name, value):
        """Filter categories with maximum product count (recursive)"""
        if not value or value < 0:
            return queryset

        from product.models.product import Product

        category_ids = []
        for category in queryset:
            descendants = category.get_descendants(include_self=True)
            product_count = Product.objects.filter(
                category__in=descendants
            ).count()
            if product_count <= value:
                category_ids.append(category.id)

        return queryset.filter(id__in=category_ids)

    def filter_has_products(self, queryset, name, value):
        """Filter categories that have products (recursive)"""
        from product.models.product import Product

        category_ids = []
        for category in queryset:
            descendants = category.get_descendants(include_self=True)
            has_products = Product.objects.filter(
                category__in=descendants
            ).exists()
            if has_products == value:
                category_ids.append(category.id)

        return queryset.filter(id__in=category_ids)

    def filter_ancestor_of(self, queryset, name, value):
        """Filter ancestors of specified category"""
        try:
            category = ProductCategory.objects.get(id=value)
            ancestors = category.get_ancestors()
            return queryset.filter(
                id__in=ancestors.values_list("id", flat=True)
            )
        except ProductCategory.DoesNotExist:
            return queryset.none()

    def filter_descendant_of(self, queryset, name, value):
        """Filter descendants of specified category"""
        try:
            category = ProductCategory.objects.get(id=value)
            descendants = category.get_descendants()
            return queryset.filter(
                id__in=descendants.values_list("id", flat=True)
            )
        except ProductCategory.DoesNotExist:
            return queryset.none()

    def filter_sibling_of(self, queryset, name, value):
        """Filter siblings of specified category"""
        try:
            category = ProductCategory.objects.get(id=value)
            siblings = category.get_siblings()
            return queryset.filter(id__in=siblings.values_list("id", flat=True))
        except ProductCategory.DoesNotExist:
            return queryset.none()



from django.utils.translation import gettext_lazy as _
from rest_framework.permissions import BasePermission
from django.contrib.auth import get_user_model

User = get_user_model()


class IsOwnerOrAdmin(BasePermission):
    message = _("You do not have permission to access this object.")

    def has_permission(self, request, view):
        return bool(request.user and request.user.is_authenticated)

    def has_object_permission(self, request, view, obj):
        if request.user.is_staff:
            return True

        return self._is_owner(request.user, obj)

    def _is_owner(self, user, obj):
        if hasattr(obj, "user"):
            return obj.user == user

        if self._is_user_instance(obj):
            return obj.id == user.id

        if hasattr(obj, "owner"):
            return obj.owner == user

        if hasattr(obj, "created_by"):
            return obj.created_by == user

        return False

    def _is_user_instance(self, obj):
        return isinstance(obj, User)



from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase
from djmoney.money import Money

from country.factories import CountryFactory
from order.enum.status import OrderStatus
from order.factories.order import OrderFactory
from order.serializers.item import OrderItemDetailSerializer
from order.serializers.order import (
    OrderDetailSerializer,
    OrderWriteSerializer,
)
from pay_way.factories import PayWayFactory
from product.factories.product import ProductFactory
from region.factories import RegionFactory

User = get_user_model()


class OrderDetailSerializerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            email="test2@example.com",
            password="testpassword",
        )

        self.pay_way = PayWayFactory()
        self.country = CountryFactory()
        self.region = RegionFactory(country=self.country)

        self.order = OrderFactory(
            user=self.user,
            pay_way=self.pay_way,
            country=self.country,
            region=self.region,
            status=OrderStatus.PENDING.value,
        )

        product = ProductFactory(
            name="Test Product", price=Money("50.00", settings.DEFAULT_CURRENCY)
        )
        self.order_item = self.order.items.create(
            product=product,
            price=Money("50.00", settings.DEFAULT_CURRENCY),
            quantity=2,
        )

        self.serializer = OrderDetailSerializer(instance=self.order)

    def test_contains_expected_fields(self):
        data = self.serializer.data

        expected_additional_fields = {
            "order_timeline",
            "pricing_breakdown",
            "tracking_details",
        }

        self.assertTrue(
            all(field in data for field in expected_additional_fields)
        )


class OrderCreateUpdateSerializerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            email="test3@example.com",
            password="testpassword",
        )

        self.pay_way = PayWayFactory()
        self.country = CountryFactory()
        self.region = RegionFactory(country=self.country)

        self.product = ProductFactory(
            stock=10, price=Money("50.00", settings.DEFAULT_CURRENCY)
        )

        self.valid_data = {
            "email": "customer@example.com",
            "first_name": "John",
            "last_name": "Doe",
            "phone": "+12025550195",
            "paid_amount": {
                "amount": "100.00",
                "currency": settings.DEFAULT_CURRENCY,
            },
            "status": OrderStatus.PENDING.value,
            "shipping_price": {
                "amount": "10.00",
                "currency": settings.DEFAULT_CURRENCY,
            },
            "street": "Main Street",
            "street_number": "123",
            "city": "Testville",
            "zipcode": "12345",
            "country": self.country.alpha_2,
            "region": self.region.alpha,
            "pay_way": self.pay_way.id,
            "items": [
                {
                    "product": self.product.id,
                    "quantity": 2,
                    "price": {
                        "amount": "50.00",
                        "currency": settings.DEFAULT_CURRENCY,
                    },
                }
            ],
        }

    def test_items_validation(self):
        invalid_data = self.valid_data.copy()
        invalid_data["items"] = []

        serializer = OrderWriteSerializer(data=invalid_data)
        self.assertFalse(serializer.is_valid())

        self.assertFalse(serializer.is_valid())

    def test_money_field_validation(self):
        invalid_data = self.valid_data.copy()
        invalid_data["paid_amount"] = {
            "amount": "-50.00",
            "currency": settings.DEFAULT_CURRENCY,
        }

        serializer = OrderWriteSerializer(data=invalid_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("paid_amount", serializer.errors)

        invalid_data = self.valid_data.copy()
        invalid_data["paid_amount"] = {
            "amount": "50.00",
            "currency": "INVALID",
        }

        serializer = OrderWriteSerializer(data=invalid_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("paid_amount", serializer.errors)

    def test_phone_number_validation(self):
        invalid_data = self.valid_data.copy()
        invalid_data["phone"] = "not-a-phone-number"

        serializer = OrderWriteSerializer(data=invalid_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("phone", serializer.errors)

    def test_required_fields(self):
        invalid_data = self.valid_data.copy()
        del invalid_data["email"]

        serializer = OrderWriteSerializer(data=invalid_data)
        serializer.is_valid()
        self.assertIn("email", serializer.errors)

        invalid_data = self.valid_data.copy()
        del invalid_data["first_name"]

        serializer = OrderWriteSerializer(data=invalid_data)
        serializer.is_valid()
        self.assertIn("first_name", serializer.errors)

    def test_email_validation(self):
        invalid_data = self.valid_data.copy()
        invalid_data["email"] = "not-an-email"

        serializer = OrderWriteSerializer(data=invalid_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)


class OrderItemSerializerTestCase(TestCase):
    def setUp(self):
        self.product = ProductFactory(
            name="Test Product", price=Money("50.00", settings.DEFAULT_CURRENCY)
        )
        self.order = OrderFactory()
        self.order_item = self.order.items.create(
            product=self.product,
            price=Money("50.00", settings.DEFAULT_CURRENCY),
            quantity=2,
        )

        self.serializer = OrderItemDetailSerializer(instance=self.order_item)

    def test_contains_expected_fields(self):
        data = self.serializer.data

        expected_fields = {
            "id",
            "price",
            "product",
            "quantity",
            "original_quantity",
            "is_refunded",
            "refunded_quantity",
            "net_quantity",
            "sort_order",
            "created_at",
            "updated_at",
            "uuid",
            "total_price",
            "refunded_amount",
            "net_price",
            "notes",
        }

        self.assertTrue(all(field in data for field in expected_fields))



# Generated by Django 5.2.1 on 2025-05-14 20:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('vat', '0005_alter_vat_value_vat_vat_value_range'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='vat',
            new_name='vat_created_at_ix',
            old_name='vat_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='vat',
            new_name='vat_updated_at_ix',
            old_name='vat_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='vat',
            new_name='vat_value_ix',
            old_name='vat_vat_value_d54579_btree',
        ),
    ]



from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from core.filters.camel_case_filters import CamelCaseTimeStampFilterSet
from core.filters.core import (
    SortableFilterMixin,
    UUIDFilterMixin,
)
from region.models import Region


class RegionFilter(
    SortableFilterMixin, UUIDFilterMixin, CamelCaseTimeStampFilterSet
):
    alpha = filters.CharFilter(
        field_name="alpha",
        lookup_expr="icontains",
        help_text=_("Filter by region alpha code (partial match)"),
    )
    alpha_exact = filters.CharFilter(
        field_name="alpha",
        lookup_expr="exact",
        help_text=_("Filter by exact region alpha code"),
    )
    country = filters.CharFilter(
        field_name="country__alpha_2",
        lookup_expr="exact",
        help_text=_("Filter by country alpha-2 code"),
    )
    country_name = filters.CharFilter(
        field_name="country__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by country name (partial match)"),
    )
    name = filters.CharFilter(
        field_name="translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by region name (partial match)"),
    )

    class Meta:
        model = Region
        fields = {
            "alpha": ["exact", "icontains"],
            "country": ["exact"],
            "sort_order": ["exact", "gte", "lte"],
            "uuid": ["exact"],
        }



import logging
from typing import Any

from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from djstripe.event_handlers import djstripe_receiver
from djstripe.models import Event
from order.enum.document_type import OrderDocumentTypeEnum
from order.enum.status import OrderStatus, PaymentStatus
from order.models.history import OrderHistory, OrderItemHistory
from order.models.item import OrderItem
from order.models.order import Order
from order.notifications import (
    send_order_canceled_notification,
    send_order_delivered_notification,
    send_order_shipped_notification,
)
from order.services import OrderService
from order.signals import (
    order_canceled,
    order_completed,
    order_created,
    order_delivered,
    order_paid,
    order_refunded,
    order_returned,
    order_shipped,
    order_status_changed,
)
from order.tasks import (
    generate_order_invoice,
    send_order_confirmation_email,
    send_order_status_update_email,
    send_shipping_notification_email,
)

logger = logging.getLogger(__name__)


@receiver(post_save, sender=Order)
def handle_order_post_save(
    sender: type[Order], instance: Order, created: bool, **kwargs: Any
) -> None:
    if created:
        order_created.send(sender=sender, order=instance)
        logger.debug(f"Sent order_created signal for new order {instance.id}")
        return

    if (
        hasattr(instance, "_previous_status")
        and instance._previous_status != instance.status
    ):
        order_status_changed.send(
            sender=sender,
            order=instance,
            old_status=instance._previous_status,
            new_status=instance.status,
        )
        logger.debug(
            f"Sent order_status_changed signal for order {instance.id} "
            f"({instance._previous_status} -> {instance.status})"
        )


@receiver(order_created)
def handle_order_created(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    send_order_confirmation_email.delay(order.id)

    OrderHistory.log_note(
        order=order,
        note="Order created",
    )


@receiver(order_status_changed)
def handle_order_status_changed(
    sender: type[Order],
    order: Order,
    old_status: str,
    new_status: str | None = None,
    **kwargs: Any,
) -> None:
    if new_status is None:
        new_status = order.status

    OrderHistory.log_status_change(
        order=order, previous_status=old_status, new_status=new_status
    )

    send_order_status_update_email.delay(order.id, new_status)

    if new_status == OrderStatus.SHIPPED.value:
        order_shipped.send(sender=sender, order=order)

    elif new_status == OrderStatus.DELIVERED.value:
        order_delivered.send(sender=sender, order=order)

    elif new_status == OrderStatus.CANCELED.value:
        order_canceled.send(sender=sender, order=order)

    elif new_status == OrderStatus.COMPLETED.value:
        order_completed.send(sender=sender, order=order)

    elif (
        new_status == OrderStatus.PROCESSING.value
        and order.is_paid
        and not hasattr(order, "_paid_signal_sent")
    ):
        order_paid.send(sender=sender, order=order)
        order._paid_signal_sent = True
        logger.debug(f"Sent order_paid signal for order {order.id}")

    logger.info(
        f"Order {order.id} status changed from {old_status} to {new_status}"
    )


@receiver(pre_save, sender=Order)
def handle_order_pre_save(
    sender: type[Order], instance: Order, **kwargs: Any
) -> None:
    try:
        if instance.pk:
            instance._previous_status = Order.objects.get(pk=instance.pk).status
        else:
            instance._previous_status = None
    except Order.DoesNotExist:
        instance._previous_status = None


@receiver(pre_save, sender=OrderItem)
def handle_order_item_pre_save(
    sender: Any, instance: Any, **kwargs: Any
) -> None:
    if instance.pk:
        try:
            original = sender.objects.get(pk=instance.pk)

            instance._original_quantity = original.quantity
            instance._original_price = original.price
            instance._original_is_refunded = original.is_refunded
            instance._original_refunded_quantity = original.refunded_quantity

        except sender.DoesNotExist:
            instance._original_quantity = 0
            instance._original_price = None
        except Exception as e:
            logger.error(
                f"Error in handle_order_item_pre_save: {e}", exc_info=True
            )
    else:
        instance._original_quantity = 0
        instance._original_price = None


@receiver(post_save, sender=OrderItem)
def handle_order_item_post_save(
    sender: type[OrderItem], instance: OrderItem, created: bool, **kwargs: Any
) -> None:
    if created:
        product = instance.product
        product.stock = max(0, product.stock - instance.quantity)
        product.save(update_fields=["stock"])

        try:
            order = instance.order

            OrderHistory.log_note(
                order=order,
                note=f"Item {instance.product.safe_translation_getter('name', any_language=True) if instance.product else 'Unknown'} added to order",
            )
            logger.debug(
                f"Order item {instance.id} created for order {order.id}"
            )
        except Exception as e:
            logger.error(
                f"Error handling order item creation: {e}", exc_info=True
            )
    elif (
        hasattr(instance, "_original_quantity")
        and instance._original_quantity != instance.quantity
    ):
        product = instance.product
        stock_difference = instance._original_quantity - instance.quantity
        new_stock = product.stock + stock_difference
        product.stock = max(0, new_stock)
        product.save(update_fields=["stock"])

        OrderItemHistory.log_quantity_change(
            order_item=instance,
            previous_quantity=instance._original_quantity,
            new_quantity=instance.quantity,
        )

        try:
            OrderHistory.log_note(
                order=instance.order,
                note=f"Item {instance.product.safe_translation_getter('name', any_language=True) if instance.product else 'Unknown'} quantity updated from {instance._original_quantity} to {instance.quantity}",
            )
        except Exception as e:
            logger.error(
                f"Error logging order history for quantity change: {e}",
                exc_info=True,
            )

    if (
        hasattr(instance, "_original_price")
        and instance._original_price
        and instance._original_price != instance.price
    ):
        OrderItemHistory.log_price_update(
            order_item=instance,
            previous_price=instance._original_price,
            new_price=instance.price,
        )

    if (
        hasattr(instance, "_original_is_refunded")
        and instance.is_refunded != instance._original_is_refunded
    ):
        try:
            OrderHistory.log_note(
                order=instance.order,
                note=(
                    f"Item {instance.product.safe_translation_getter('name', any_language=True) if instance.product else 'Unknown'} "
                    f"marked as {'refunded' if instance.is_refunded else 'not refunded'}"
                ),
            )
        except Exception as e:
            logger.error(
                f"Error logging order history for refund change: {e}",
                exc_info=True,
            )


@receiver(order_shipped)
def handle_order_shipped(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    if order.status != OrderStatus.SHIPPED.value:
        logger.info(f"Updating order {order.id} status to shipped")
        OrderService.update_order_status(order, OrderStatus.SHIPPED)

    OrderHistory.log_shipping_update(
        order=order,
        previous_value={"status": OrderStatus.PENDING.value},
        new_value={"status": OrderStatus.SHIPPED.value},
    )

    send_order_shipped_notification(order)

    task = send_shipping_notification_email.delay(order.id)
    logger.info(
        f"Order {order.id} shipment notification email queued (task_id: {task.id})"
    )


@receiver(order_delivered)
def handle_order_delivered(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    send_order_delivered_notification(order)

    OrderHistory.log_shipping_update(
        order=order,
        previous_value={"status": OrderStatus.SHIPPED.value},
        new_value={"status": OrderStatus.DELIVERED.value},
    )


@receiver(order_canceled)
def handle_order_canceled(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    previous_status = kwargs.get("previous_status")

    try:
        cancellation_reason = kwargs.get("reason")
        if cancellation_reason:
            OrderHistory.log_note(
                order=order,
                note=f"Order canceled. Reason: {cancellation_reason}",
            )

        send_order_canceled_notification(order)

        logger.info(
            f"Order {order.id} canceled (previous status: {previous_status})"
        )

    except Exception as e:
        logger.error(
            f"Error handling order_canceled signal: {e}", exc_info=True
        )


@receiver(order_completed)
def handle_order_completed(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    try:
        if order.document_type == OrderDocumentTypeEnum.INVOICE.value:
            task = generate_order_invoice.delay(order.id)
            logger.info(
                f"Invoice generation queued for order {order.id} (task_id: {task.id})"
            )

        OrderHistory.log_note(
            order=order,
            note="Order completed",
        )

        logger.info(f"Order {order.id} marked as completed")

    except Exception as e:
        logger.error(
            f"Error handling order_completed signal: {e}", exc_info=True
        )


@receiver(order_refunded)
def handle_order_refunded(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    try:
        refund_amount = kwargs.get("amount")
        refund_reason = kwargs.get("reason", "")

        OrderHistory.log_refund(
            order=order,
            refund_data={
                "amount": str(refund_amount)
                if refund_amount
                else "Full order amount",
                "reason": refund_reason or "Not specified",
            },
        )

        logger.info(f"Order {order.id} refunded")

    except Exception as e:
        logger.error(
            f"Error handling order_refunded signal: {e}", exc_info=True
        )


@receiver(order_returned)
def handle_order_returned(
    sender: type[Order], order: Order, **kwargs: Any
) -> None:
    try:
        return_reason = kwargs.get("reason", "")
        return_items = kwargs.get("items", [])

        items_text = ""
        if return_items:
            items_text = "Items: " + ", ".join(
                f"{item.get('product_name', 'Unknown')} (qty: {item.get('quantity', 0)})"
                for item in return_items
            )

        OrderHistory.log_note(
            order=order,
            note=(
                f"Order returned. "
                f"{items_text}. "
                f"Reason: {return_reason or 'Not specified'}"
            ),
        )

        logger.info(f"Order {order.id} returned")

    except Exception as e:
        logger.error(
            f"Error handling order_returned signal: {e}", exc_info=True
        )


@djstripe_receiver("payment_intent.succeeded")
def handle_stripe_payment_succeeded(sender, **kwargs):
    print("Handling stripe payment succeeded")
    try:
        event: Event = kwargs.get("event")
        payment_intent_data = event.data["object"]
        payment_intent_id = payment_intent_data["id"]

        logger.info(f"Stripe payment succeeded: {payment_intent_id}")

        try:
            order = Order.objects.get(payment_id=payment_intent_id)
        except Order.DoesNotExist:
            logger.error(
                f"Order not found for payment_intent: {payment_intent_id}"
            )
            return

        order.mark_as_paid(
            payment_id=payment_intent_id, payment_method="stripe"
        )

        if order.status == OrderStatus.PENDING:
            OrderService.update_order_status(order, OrderStatus.PROCESSING)

        OrderHistory.log_payment_update(
            order=order,
            previous_value={"payment_status": "pending"},
            new_value={
                "payment_status": "completed",
                "payment_id": payment_intent_id,
            },
        )

        logger.info(f"Order {order.id} marked as paid successfully")

    except Exception as e:
        logger.error(
            f"Error handling payment_intent.succeeded: {e}", exc_info=True
        )


@djstripe_receiver("payment_intent.payment_failed")
def handle_stripe_payment_failed(sender, **kwargs):
    print("Handling stripe payment failed")
    try:
        event: Event = kwargs.get("event")
        payment_intent_data = event.data["object"]
        payment_intent_id = payment_intent_data["id"]

        logger.info(f"Stripe payment failed: {payment_intent_id}")

        try:
            order = Order.objects.get(payment_id=payment_intent_id)
        except Order.DoesNotExist:
            logger.error(
                f"Order not found for payment_intent: {payment_intent_id}"
            )
            return

        order.payment_status = PaymentStatus.FAILED
        order.save(update_fields=["payment_status"])

        OrderHistory.log_payment_update(
            order=order,
            previous_value={"payment_status": "pending"},
            new_value={
                "payment_status": "failed",
                "payment_id": payment_intent_id,
            },
        )

        logger.info(f"Order {order.id} payment marked as failed")

    except Exception as e:
        logger.error(
            f"Error handling payment_intent.payment_failed: {e}", exc_info=True
        )


@djstripe_receiver("payment_intent.requires_action")
def handle_stripe_payment_requires_action(sender, **kwargs):
    print("Handling stripe payment requires action")
    try:
        event: Event = kwargs.get("event")
        payment_intent_data = event.data["object"]
        payment_intent_id = payment_intent_data["id"]

        logger.info(f"Stripe payment requires action: {payment_intent_id}")

        try:
            order = Order.objects.get(payment_id=payment_intent_id)
        except Order.DoesNotExist:
            logger.error(
                f"Order not found for payment_intent: {payment_intent_id}"
            )
            return

        order.payment_status = PaymentStatus.PENDING
        order.save(update_fields=["payment_status"])

        OrderHistory.log_note(
            order=order,
            note=f"Payment requires additional action (3D Secure, etc.) - Payment ID: {payment_intent_id}",
        )

    except Exception as e:
        logger.error(
            f"Error handling payment_intent.requires_action: {e}", exc_info=True
        )


@djstripe_receiver("charge.dispute.created")
def handle_stripe_dispute_created(sender, **kwargs):
    print("Handling stripe dispute created")
    try:
        event: Event = kwargs.get("event")
        dispute_data = event.data["object"]
        charge_id = dispute_data["charge"]

        logger.warning(f"Stripe dispute created for charge: {charge_id}")

        # Try to find the order by looking for the charge
        # This might require additional logic depending on setup
        # For now, just log the dispute for manual review

    except Exception as e:
        logger.error(
            f"Error handling charge.dispute.created: {e}", exc_info=True
        )


@djstripe_receiver("checkout.session.completed")
def handle_stripe_checkout_completed(sender, **kwargs):
    print("Handling stripe checkout session completed")
    try:
        event: Event = kwargs.get("event")
        session_data = event.data["object"]
        session_id = session_data["id"]
        payment_intent_id = session_data.get("payment_intent")
        payment_status = session_data.get("payment_status")

        logger.info(f"Checkout session completed: {session_id}")

        order_id = session_data.get("metadata", {}).get("order_id")

        if not order_id:
            logger.warning(f"No order_id in session metadata: {session_id}")
            return

        try:
            order = Order.objects.get(id=order_id)
        except Order.DoesNotExist:
            logger.error(f"Order {order_id} not found for session {session_id}")
            return

        if payment_status == "paid" and payment_intent_id:
            order.mark_as_paid(
                payment_id=payment_intent_id, payment_method="stripe"
            )

            if order.status == OrderStatus.PENDING:
                OrderService.update_order_status(order, OrderStatus.PROCESSING)

            if not order.metadata:
                order.metadata = {}
            order.metadata["stripe_checkout_session_id"] = session_id
            order.metadata["stripe_payment_intent_id"] = payment_intent_id
            order.save(update_fields=["metadata"])

            OrderHistory.log_payment_update(
                order=order,
                previous_value={"payment_status": "pending"},
                new_value={
                    "payment_status": "completed",
                    "payment_id": payment_intent_id,
                    "checkout_session_id": session_id,
                },
            )

            logger.info(
                f"Order {order_id} marked as paid via checkout session {session_id}"
            )

        elif payment_status == "unpaid":
            order.payment_status = PaymentStatus.PENDING
            order.save(update_fields=["payment_status"])

            OrderHistory.log_note(
                order=order,
                note=f"Checkout session completed but payment is unpaid: {session_id}",
            )

            logger.warning(
                f"Checkout session completed but payment is unpaid: {session_id}"
            )

    except Exception as e:
        logger.error(
            f"Error handling checkout.session.completed: {e}", exc_info=True
        )


@djstripe_receiver("checkout.session.expired")
def handle_stripe_checkout_expired(sender, **kwargs):
    print("Handling stripe checkout session expired")
    try:
        event: Event = kwargs.get("event")
        session_data = event.data["object"]
        session_id = session_data["id"]
        order_id = session_data.get("metadata", {}).get("order_id")

        logger.info(f"Checkout session expired: {session_id}")

        if not order_id:
            return

        try:
            order = Order.objects.get(id=order_id)

            if not order.is_paid:
                if not order.metadata:
                    order.metadata = {}
                order.metadata["stripe_checkout_expired"] = True
                order.metadata["stripe_checkout_session_id"] = session_id
                order.save(update_fields=["metadata"])

                OrderHistory.log_note(
                    order=order,
                    note=f"Checkout session expired: {session_id}",
                )

                logger.info(
                    f"Marked order {order_id} checkout session as expired"
                )

        except Order.DoesNotExist:
            logger.error(
                f"Order {order_id} not found for expired session {session_id}"
            )

    except Exception as e:
        logger.error(
            f"Error handling checkout.session.expired: {e}", exc_info=True
        )



# Generated by Django 5.0.4 on 2024-04-11 17:27
import django.db.models.manager
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0009_alter_blogtag_managers"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="blogcategory",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Blog Category",
                "verbose_name_plural": "Blog Categories",
            },
        ),
        migrations.AlterModelOptions(
            name="blogtag",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Blog Tag",
                "verbose_name_plural": "Blog Tags",
            },
        ),
        migrations.AlterModelManagers(
            name="blogtag",
            managers=[
                ("active_tags", django.db.models.manager.Manager()),
            ],
        ),
    ]



import importlib
from datetime import timedelta

from django.core.management.base import BaseCommand
from django.utils import timezone


class Command(BaseCommand):
    help = "Delete or archive expired notifications"

    def add_arguments(self, parser):
        parser.add_argument(
            "--days",
            type=int,
            default=None,
            help="Delete notifications older than this number of days",
        )

    def handle(self, *args, **options):
        notification = importlib.import_module(
            "notification.models.notification"
        ).Notification

        days = options.get("days")

        if days is not None:
            cutoff_date = timezone.now() - timedelta(days=days)
            expired_notifications = notification.objects.filter(
                created_at__lt=cutoff_date
            )
            message = f"notifications older than {days} days"
        else:
            expired_notifications = notification.objects.filter(
                expiry_date__lt=timezone.now()
            ).exclude(expiry_date__isnull=True)
            message = "expired notifications"

        count = expired_notifications.count()

        if count == 0:
            self.stdout.write(
                self.style.WARNING(f"No {message} found to delete")
            )
            return

        deleted_count, deleted_details = expired_notifications.delete()

        self.stdout.write(
            self.style.SUCCESS(
                f"Successfully deleted {deleted_count} {message}"
            )
        )



# Generated by Django 5.0.3 on 2024-03-31 13:31
import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("vat", "0002_vat_vat_created_at_idx_vat_vat_updated_at_idx"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="vat",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["value"], name="vat_vat_value_d54579_btree"
            ),
        ),
    ]



import json
import re
from collections.abc import Mapping
from typing import Any, NotRequired, TypedDict

from django.db import models
from django.utils.functional import Promise
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from parler_rest.fields import TranslatedFieldsField
from rest_framework import serializers
from drf_spectacular.utils import OpenApiParameter


class TranslatedFieldExtended(TranslatedFieldsField):
    def to_internal_value(self, data):
        if data is None:
            return {}
        if isinstance(data, str):
            data = json.loads(data)
        if not isinstance(data, dict):
            self.fail("invalid")
        if not self.allow_empty and len(data) == 0:
            self.fail("empty")

        result, errors = {}, {}
        for lang_code, model_fields in data.items():
            serializer = self.serializer_class(data=model_fields)
            if serializer.is_valid():
                result[lang_code] = serializer.validated_data
            else:
                errors[lang_code] = serializer.errors

        if errors:
            raise serializers.ValidationError(errors)
        return result


def camel_to_words(name):
    result = re.sub(r"(?<!^)(?=[A-Z])", " ", name)
    return result


class DisplayConfig(TypedDict):
    tag: NotRequired[str]
    display_name: NotRequired[str]
    display_name_plural: NotRequired[str]


type RequestSerializersConfig = dict[str, type[serializers.Serializer]]
type ResponseSerializersConfig = dict[str, type[serializers.Serializer]]


def create_schema_view_config(
    model_class: type[models.Model],
    request_serializers: RequestSerializersConfig | None = None,
    response_serializers: ResponseSerializersConfig | None = None,
    error_serializer=None,
    additional_responses=None,
    display_config: DisplayConfig | Mapping[str, Any] | None = None,
    include_language_param: bool = True,
    include_pagination_params: bool = True,
):
    """
    Create schema configuration for a ViewSet with translation support.

    Args:
        model_class: The Django model class
        request_serializers: Dict with keys: create, update, partial_update
        response_serializers: Dict with keys: create, list, retrieve, update, partial_update, destroy
        error_serializer: Serializer for error responses
        additional_responses: Dict with additional responses per action
        display_config: DisplayConfig or dict with optional keys:
            - tag: Override for OpenAPI tag (default: model's verbose_name_plural)
            - display_name: Override for singular name (default: model's verbose_name)
            - display_name_plural: Override for plural name (default: model's verbose_name_plural)
        include_language_param: Whether to include the language query parameter for translation-enabled models
        include_pagination_params: Whether to include pagination query parameters for list endpoints

    Note:
        This function fully supports Django translations. If your model uses
        gettext_lazy for verbose_name/verbose_name_plural, all generated
        summaries and descriptions will be properly translated.
    """
    if request_serializers is None:
        request_serializers = {}
    if response_serializers is None:
        response_serializers = {}
    if additional_responses is None:
        additional_responses = {}
    if display_config is None:
        display_config = {}

    model_name = model_class.__name__

    display_name = display_config.get("display_name")
    if display_name is None:
        if hasattr(model_class._meta, "verbose_name"):
            display_name = model_class._meta.verbose_name
        else:
            display_name = camel_to_words(model_name).lower()

    display_name_plural = display_config.get("display_name_plural")
    if display_name_plural is None:
        if hasattr(model_class._meta, "verbose_name_plural"):
            display_name_plural = model_class._meta.verbose_name_plural
        elif isinstance(display_name, Promise):
            display_name_plural = display_name
        else:
            display_name_plural = f"{display_name}s"

    tag = display_config.get("tag")
    if tag is None:
        if hasattr(model_class._meta, "verbose_name_plural"):
            tag = model_class._meta.verbose_name_plural
        else:
            tag = camel_to_words(model_name)

    base_error_responses = (
        {
            400: error_serializer,
            401: error_serializer,
            403: error_serializer,
            404: error_serializer,
            500: error_serializer,
        }
        if error_serializer
        else {}
    )

    language_parameter = None
    if include_language_param:
        from django.conf import settings

        try:
            available_languages = [
                lang["code"]
                for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
            ]
            default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE
            language_parameter = OpenApiParameter(
                name="language",
                description=_("Language code for translations (%s)")
                % ", ".join(available_languages),
                required=False,
                type=str,
                enum=available_languages,
                default=default_language,
            )
        except (AttributeError, KeyError):
            language_parameter = None

    pagination_parameters = []
    if include_pagination_params:
        pagination_type_parameter = OpenApiParameter(
            name="pagination_type",
            description=_("Pagination strategy type"),
            required=False,
            type=str,
            enum=["pageNumber", "cursor", "limitOffset"],
            default="pageNumber",
        )

        pagination_parameter = OpenApiParameter(
            name="pagination",
            description=_("Enable or disable pagination"),
            required=False,
            type=str,
            enum=["true", "false"],
            default="true",
        )

        page_size_parameter = OpenApiParameter(
            name="page_size",
            description=_("Number of results to return per page"),
            required=False,
            type=int,
            default=20,
        )

        pagination_parameters = [
            pagination_type_parameter,
            pagination_parameter,
            page_size_parameter,
        ]

    list_parameters = []
    if language_parameter:
        list_parameters.append(language_parameter)
    list_parameters.extend(pagination_parameters)

    create_request_serializer = request_serializers.get("create")
    update_request_serializer = request_serializers.get("update")
    partial_update_request_serializer = request_serializers.get(
        "partial_update"
    )

    create_response_serializer = response_serializers.get("create")
    list_response_serializer = response_serializers.get("list")
    retrieve_response_serializer = response_serializers.get("retrieve")
    update_response_serializer = response_serializers.get("update")
    partial_update_response_serializer = response_serializers.get(
        "partial_update"
    )
    destroy_response_serializer = response_serializers.get("destroy")

    config = {
        "list": extend_schema(
            operation_id=f"list{model_name}",
            summary=_("List %(name)s") % {"name": display_name_plural},
            description=_(
                "Retrieve a list of %(name)s with filtering and search capabilities. "
                "Supports multiple pagination strategies: pageNumber (default), cursor, and limitOffset."
            )
            % {"name": display_name_plural},
            tags=[tag],
            parameters=list_parameters,
            responses={
                200: list_response_serializer(many=True)
                if list_response_serializer
                else None,
                **base_error_responses,
                **additional_responses.get("list", {}),
            },
        ),
        "create": extend_schema(
            operation_id=f"create{model_name}",
            summary=_("Create a %(name)s") % {"name": display_name},
            description=_("Create a new %(name)s. Requires authentication.")
            % {"name": display_name},
            tags=[tag],
            parameters=[language_parameter] if language_parameter else None,
            request=create_request_serializer,
            responses={
                201: create_response_serializer,
                **base_error_responses,
                **additional_responses.get("create", {}),
            },
        ),
        "retrieve": extend_schema(
            operation_id=f"retrieve{model_name}",
            summary=_("Retrieve a %(name)s") % {"name": display_name},
            description=_("Get detailed information about a specific %(name)s.")
            % {"name": display_name},
            tags=[tag],
            parameters=[language_parameter] if language_parameter else None,
            responses={
                200: retrieve_response_serializer,
                **{k: v for k, v in base_error_responses.items() if k != 400},
                **additional_responses.get("retrieve", {}),
            },
        ),
        "update": extend_schema(
            operation_id=f"update{model_name}",
            summary=_("Update a %(name)s") % {"name": display_name},
            description=_(
                "Update %(name)s information. Requires authentication."
            )
            % {"name": display_name},
            tags=[tag],
            parameters=[language_parameter] if language_parameter else None,
            request=update_request_serializer,
            responses={
                200: update_response_serializer,
                **base_error_responses,
                **additional_responses.get("update", {}),
            },
        ),
        "partial_update": extend_schema(
            operation_id=f"partialUpdate{model_name}",
            summary=_("Partially update a %(name)s") % {"name": display_name},
            description=_(
                "Partially update %(name)s information. Requires authentication."
            )
            % {"name": display_name},
            tags=[tag],
            parameters=[language_parameter] if language_parameter else None,
            request=partial_update_request_serializer,
            responses={
                200: partial_update_response_serializer,
                **base_error_responses,
                **additional_responses.get("partial_update", {}),
            },
        ),
        "destroy": extend_schema(
            operation_id=f"destroy{model_name}",
            summary=_("Delete a %(name)s") % {"name": display_name},
            description=_("Delete a %(name)s. Requires authentication.")
            % {"name": display_name},
            tags=[tag],
            responses={
                204: destroy_response_serializer,
                **{
                    k: v
                    for k, v in base_error_responses.items()
                    if k not in [400, 500]
                },
                **additional_responses.get("destroy", {}),
            },
        ),
    }

    return config



from datetime import timedelta
from django.utils import timezone
from django.urls import reverse
from django.test import TransactionTestCase
from rest_framework.test import APIClient
import pytest

from blog.factories.category import BlogCategoryFactory
from blog.factories.post import BlogPostFactory
from blog.models.category import BlogCategory


@pytest.mark.django_db(transaction=True)
class BlogCategoryFilterTest(TransactionTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()

    def setUp(self):
        BlogCategory.objects.all().delete()

        self.client = APIClient()

        self.now = timezone.now()

        self.root1 = BlogCategoryFactory(
            parent=None, slug="root1", sort_order=1
        )
        self.root1.created_at = self.now - timedelta(days=90)
        self.root1.save()
        self.root1.set_current_language("en")
        self.root1.name = "Technology"
        self.root1.description = "All about tech and gadgets"
        self.root1.save()

        self.root2 = BlogCategoryFactory(
            parent=None,
            slug="root2",
            sort_order=2,
            image="uploads/blog/root2.jpg",
        )
        self.root2.created_at = self.now - timedelta(days=60)
        self.root2.save()
        self.root2.set_current_language("en")
        self.root2.name = "Travel"
        self.root2.description = "Travel guides and tips"
        self.root2.save()

        self.root3 = BlogCategoryFactory(
            parent=None, slug="root3", sort_order=3
        )
        self.root3.created_at = self.now - timedelta(days=30)
        self.root3.save()
        self.root3.set_current_language("en")
        self.root3.name = "Lifestyle"
        self.root3.description = "Lifestyle and wellness"
        self.root3.save()

        self.child1_1 = BlogCategoryFactory(
            parent=self.root1,
            slug="child1_1",
            sort_order=1,
            image="uploads/blog/child1_1.jpg",
        )
        self.child1_1.created_at = self.now - timedelta(days=45)
        self.child1_1.save()
        self.child1_1.set_current_language("en")
        self.child1_1.name = "Software"
        self.child1_1.description = "Software development and programming"
        self.child1_1.save()

        self.child1_2 = BlogCategoryFactory(
            parent=self.root1, slug="child1_2", sort_order=2
        )
        self.child1_2.created_at = self.now - timedelta(days=40)
        self.child1_2.save()
        self.child1_2.set_current_language("en")
        self.child1_2.name = "Hardware"
        self.child1_2.description = "Computer hardware and components"
        self.child1_2.save()

        self.child2_1 = BlogCategoryFactory(
            parent=self.root2, slug="child2_1", sort_order=1
        )
        self.child2_1.created_at = self.now - timedelta(days=20)
        self.child2_1.save()
        self.child2_1.set_current_language("en")
        self.child2_1.name = "Europe"
        self.child2_1.description = "European travel destinations"
        self.child2_1.save()

        self.grandchild1_1_1 = BlogCategoryFactory(
            parent=self.child1_1, slug="grandchild1_1_1", sort_order=1
        )
        self.grandchild1_1_1.created_at = self.now - timedelta(days=10)
        self.grandchild1_1_1.save()
        self.grandchild1_1_1.set_current_language("en")
        self.grandchild1_1_1.name = "Python"
        self.grandchild1_1_1.description = "Python programming language"
        self.grandchild1_1_1.save()

        BlogCategory.objects.rebuild()

        for i in range(2):
            BlogPostFactory(category=self.root2, is_published=True)

        for i in range(3):
            BlogPostFactory(category=self.child1_1, is_published=True)

        BlogPostFactory(category=self.child2_1, is_published=True)

        for i in range(4):
            BlogPostFactory(category=self.grandchild1_1_1, is_published=True)

    def test_timestamp_filters(self):
        url = reverse("blog-category-list")

        created_after = self.now - timedelta(days=50)
        response = self.client.get(
            url, {"created_after": created_after.isoformat()}
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root3.id, result_ids)
        self.assertIn(self.child1_1.id, result_ids)
        self.assertIn(self.child1_2.id, result_ids)
        self.assertIn(self.child2_1.id, result_ids)
        self.assertIn(self.grandchild1_1_1.id, result_ids)

        created_before = self.now - timedelta(days=35)
        response = self.client.get(
            url, {"created_before": created_before.isoformat()}
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        expected_old_categories = [
            self.root1.id,
            self.root2.id,
            self.child1_1.id,
            self.child1_2.id,
        ]
        for cat_id in expected_old_categories:
            self.assertIn(cat_id, result_ids)

    def test_uuid_and_sort_order_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"uuid": str(self.child1_1.uuid)})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.child1_1.id, result_ids)
        category_found = any(
            r["id"] == self.child1_1.id for r in response.data["results"]
        )
        self.assertTrue(category_found)

        response = self.client.get(url, {"sort_order": 1})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_categories = [
            self.root1.id,
            self.child1_1.id,
            self.child2_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_categories:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"sort_order_min": 2})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root2.id, result_ids)
        self.assertIn(self.root3.id, result_ids)
        self.assertIn(self.child1_2.id, result_ids)

    def test_hierarchy_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"parent": self.root1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_children = [self.child1_1.id, self.child1_2.id]
        for cat_id in expected_children:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"parent__isnull": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_roots = [self.root1.id, self.root2.id, self.root3.id]
        for cat_id in expected_roots:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"parent__isnull": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_non_roots = [
            self.child1_1.id,
            self.child1_2.id,
            self.child2_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_non_roots:
            self.assertIn(cat_id, result_ids)

    def test_level_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"level": 0})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_level_0 = [self.root1.id, self.root2.id, self.root3.id]
        for cat_id in expected_level_0:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"level": 1})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_level_1 = [
            self.child1_1.id,
            self.child1_2.id,
            self.child2_1.id,
        ]
        for cat_id in expected_level_1:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"level__gte": 1})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.child1_1.id, result_ids)
        self.assertIn(self.grandchild1_1_1.id, result_ids)

        response = self.client.get(url, {"level__lte": 1})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root1.id, result_ids)
        self.assertIn(self.child1_1.id, result_ids)

    def test_content_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"name": "tech"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root1.id, result_ids)

        response = self.client.get(url, {"description": "programming"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.child1_1.id, result_ids)
        self.assertIn(self.grandchild1_1_1.id, result_ids)

        response = self.client.get(url, {"slug__icontains": "child"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_categories = [
            self.child1_1.id,
            self.child1_2.id,
            self.child2_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_categories:
            self.assertIn(cat_id, result_ids)

    def test_image_filter(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"has_image": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root2.id, result_ids)
        self.assertIn(self.child1_1.id, result_ids)

        response = self.client.get(url, {"has_image": "false"})
        self.assertEqual(response.status_code, 200)

    def test_post_count_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"has_posts": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_with_posts = [
            self.root2.id,
            self.child1_1.id,
            self.child2_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_with_posts:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"has_posts": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_without_posts = [
            self.root1.id,
            self.root3.id,
            self.child1_2.id,
        ]
        for cat_id in expected_without_posts:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"min_post_count": 2})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_min_2_posts = [
            self.root2.id,
            self.child1_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_min_2_posts:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"max_post_count": 2})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root2.id, result_ids)
        self.assertIn(self.child2_1.id, result_ids)

    def test_recursive_post_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"has_recursive_posts": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root1.id, result_ids)
        self.assertIn(self.root2.id, result_ids)
        self.assertIn(self.child1_1.id, result_ids)

        response = self.client.get(url, {"has_recursive_posts": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root3.id, result_ids)
        self.assertIn(self.child1_2.id, result_ids)

        response = self.client.get(url, {"min_recursive_post_count": 5})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root1.id, result_ids)

    def test_tree_structure_filters(self):
        url = reverse("blog-category-list")

        response = self.client.get(url, {"is_leaf": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_leaf_nodes = [
            self.root3.id,
            self.child1_2.id,
            self.child2_1.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_leaf_nodes:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"has_children": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_with_children = [
            self.root1.id,
            self.root2.id,
            self.child1_1.id,
        ]
        for cat_id in expected_with_children:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(
            url, {"ancestor_of": self.grandchild1_1_1.id}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_ancestors = [self.root1.id, self.child1_1.id]
        for cat_id in expected_ancestors:
            self.assertIn(cat_id, result_ids)

        response = self.client.get(url, {"descendant_of": self.root1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_descendants = [
            self.child1_1.id,
            self.child1_2.id,
            self.grandchild1_1_1.id,
        ]
        for cat_id in expected_descendants:
            self.assertIn(cat_id, result_ids)

    def test_camel_case_filters(self):
        url = reverse("blog-category-list")

        created_after = self.now - timedelta(days=50)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after.isoformat(),
                "hasImage": "true",
                "hasPosts": "true",
                "sortOrderMax": 2,
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.child1_1.id, result_ids)

        response = self.client.get(url, {"parentIsnull": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root1.id, result_ids)
        self.assertIn(self.root2.id, result_ids)
        self.assertIn(self.root3.id, result_ids)

    def test_complex_filter_combinations(self):
        url = reverse("blog-category-list")

        response = self.client.get(
            url, {"parentIsnull": "true", "hasPosts": "true", "level": 0}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.root2.id, result_ids)

        created_after = self.now - timedelta(days=25)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after.isoformat(),
                "description": "programming",
                "hasChildren": "false",
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.grandchild1_1_1.id, result_ids)

    def test_filter_with_ordering(self):
        url = reverse("blog-category-list")

        response = self.client.get(
            url, {"isLeaf": "true", "ordering": "-createdAt"}
        )
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]
        self.assertEqual(results[0]["id"], self.grandchild1_1_1.id)
        self.assertEqual(results[1]["id"], self.child2_1.id)
        self.assertEqual(results[2]["id"], self.root3.id)
        self.assertEqual(results[3]["id"], self.child1_2.id)

    def tearDown(self):
        BlogCategory.objects.all().delete()



import os
from decimal import Decimal

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models import Avg
from django.test import TestCase
from django.utils.html import format_html
from djmoney.money import Money

from product.enum.review import ReviewStatus
from product.factories.category import ProductCategoryFactory
from product.factories.favourite import ProductFavouriteFactory
from product.factories.image import ProductImageFactory
from product.factories.product import ProductFactory
from product.factories.review import ProductReviewFactory
from product.models.product import Product
from product.models.review import ProductReview
from user.factories.account import UserAccountFactory
from vat.factories import VatFactory

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE
User = get_user_model()


class ProductModelTestCase(TestCase):
    images: list = []
    reviews: list = []

    def setUp(self):
        self.user = UserAccountFactory(num_addresses=0)
        self.category = ProductCategoryFactory()
        self.vat = VatFactory()
        self.product = ProductFactory(
            sku="P123456",
            category=self.category,
            vat=self.vat,
            slug="sample-product",
            price=Decimal("100.00"),
            active=True,
            stock=10,
            discount_percent=Decimal("50.0"),
            view_count=10,
            weight=Decimal("5.00"),
        )

        main_product_image = ProductImageFactory(
            product=self.product,
            is_main=True,
        )

        self.images.append(main_product_image)

        non_main_product_image = ProductImageFactory(
            product=self.product,
            is_main=False,
        )

        self.images.append(non_main_product_image)

        self.favourite = ProductFavouriteFactory(
            product=self.product,
            user=self.user,
        )

        user_2 = UserAccountFactory(num_addresses=0)

        product_review_status_true = ProductReviewFactory(
            product=self.product,
            user=self.user,
            rate=5,
            status=ReviewStatus.TRUE,
        )
        self.reviews.append(product_review_status_true)

        product_review_status_false = ProductReviewFactory(
            product=self.product,
            user=user_2,
            rate=5,
            status=ReviewStatus.FALSE,
        )
        self.reviews.append(product_review_status_false)

    def test_save_method(self):
        self.product.price = Decimal("100.00")
        self.product.discount_percent = Decimal("50.0")
        self.product.vat = self.vat

        self.product.save()
        self.product.refresh_from_db()

        expected_discount_value = (
            self.product.price * self.product.discount_percent
        ) / 100
        expected_vat_value = (self.product.price * self.vat.value) / 100
        expected_final_price = (
            self.product.price + expected_vat_value - expected_discount_value
        )
        expected_price_save_percent = (
            expected_discount_value / self.product.price
        ) * 100

        self.assertEqual(self.product.discount_value, expected_discount_value)
        self.assertEqual(self.product.vat_value, expected_vat_value)
        self.assertEqual(self.product.final_price, expected_final_price)
        self.assertEqual(
            self.product.price_save_percent, expected_price_save_percent
        )

    def test_fields(self):
        self.assertEqual(self.product.sku, "P123456")
        self.assertEqual(self.product.slug, "sample-product")
        self.assertEqual(
            self.product.price, Money("100.00", settings.DEFAULT_CURRENCY)
        )
        self.assertEqual(self.product.active, True)
        self.assertEqual(self.product.stock, 10)
        self.assertEqual(self.product.discount_percent, Decimal("50.0"))
        self.assertEqual(self.product.vat, self.vat)
        self.assertEqual(self.product.view_count, 10)
        self.assertEqual(self.product.weight, Decimal("5.00"))

    def test_str_representation(self):
        self.assertEqual(
            str(self.product), self.product.safe_translation_getter("name")
        )

    def test_likes_count(self):
        self.assertEqual(self.product.likes_count, 1)

    def test_review_average(self):
        self.assertEqual(self.product.review_average, 5)

    def test_review_count(self):
        self.assertEqual(self.product.review_count, 2)

    def test_vat_percent(self):
        self.assertEqual(self.product.vat_percent, self.vat.value)

    def test_vat_value(self):
        expected_vat_value = (self.product.price.amount * self.vat.value) / 100
        self.assertEqual(self.product.vat_value.amount, expected_vat_value)

    def test_product_with_tags(self):
        product = ProductFactory(num_tags=3)
        self.assertEqual(product.tags.count(), 3)

    def test_main_image_path(self):
        main_image = self.product.images.filter(is_main=True).first()
        expected_filename = (
            f"media/uploads/products/{os.path.basename(main_image.image.name)}"
        )
        self.assertEqual(self.product.main_image_path, expected_filename)

    def test_colored_stock_property(self):
        self.product.stock = 5
        self.assertEqual(
            self.product.colored_stock,
            format_html(
                '<span style="color: #1bff00;">{}</span>', self.product.stock
            ),
        )

        self.product.stock = 0
        self.assertEqual(
            self.product.colored_stock,
            format_html(
                '<span style="color: #ff0000;">{}</span>', self.product.stock
            ),
        )


class ProductQuerySetTestCase(TestCase):
    def setUp(self):
        self.vat = VatFactory(value=Decimal("10.0"))

        self.product1 = ProductFactory(
            price=Decimal("100.00"),
            discount_percent=Decimal("20.0"),
            vat=self.vat,
            stock=10,
            active=True,
        )

        ProductReviewFactory(
            product=self.product1, rate=5, status=ReviewStatus.TRUE
        )
        ProductReviewFactory(
            product=self.product1, rate=3, status=ReviewStatus.TRUE
        )
        ProductReviewFactory(
            product=self.product1, rate=4, status=ReviewStatus.FALSE
        )

        user1 = UserAccountFactory(num_addresses=0)
        user2 = UserAccountFactory(num_addresses=0)
        ProductFavouriteFactory(product=self.product1, user=user1)
        ProductFavouriteFactory(product=self.product1, user=user2)

        self.product2 = ProductFactory(
            price=Decimal("0.00"),
            discount_percent=Decimal("0.0"),
            vat=None,
            stock=0,
            active=False,
        )

        self.product3 = ProductFactory(
            price=Decimal("50.00"),
            discount_percent=Decimal("10.0"),
            vat=None,
            stock=5,
            active=True,
        )

    def test_queryset_with_discount_value_annotation(self):
        queryset = Product.objects.with_discount_value_annotation()
        product1 = queryset.get(id=self.product1.id)
        product2 = queryset.get(id=self.product2.id)

        expected_discount_value1 = (
            self.product1.price.amount * self.product1.discount_percent
        ) / 100
        expected_discount_value2 = (
            self.product2.price.amount * self.product2.discount_percent
        ) / 100

        self.assertEqual(
            product1.discount_value_annotation, expected_discount_value1
        )
        self.assertEqual(
            product2.discount_value_annotation, expected_discount_value2
        )

    def test_queryset_with_vat_value_annotation(self):
        queryset = Product.objects.with_vat_value_annotation()
        product1 = queryset.get(id=self.product1.id)
        product2 = queryset.get(id=self.product2.id)
        product3 = queryset.get(id=self.product3.id)

        expected_vat_value1 = (
            self.product1.price.amount * self.vat.value
        ) / 100
        expected_vat_value2 = Decimal("0.00")
        expected_vat_value3 = Decimal("0.00")

        self.assertEqual(product1.vat_value_annotation, expected_vat_value1)
        self.assertEqual(product2.vat_value_annotation, expected_vat_value2)
        self.assertEqual(product3.vat_value_annotation, expected_vat_value3)

    def test_queryset_with_final_price_annotation(self):
        queryset = Product.objects.with_final_price_annotation()
        product1 = queryset.get(id=self.product1.id)

        expected_discount_value = (
            self.product1.price.amount * self.product1.discount_percent
        ) / 100
        expected_vat_value = (self.product1.price.amount * self.vat.value) / 100
        expected_final_price = (
            self.product1.price.amount
            + expected_vat_value
            - expected_discount_value
        )

        self.assertEqual(product1.final_price_annotation, expected_final_price)

    def test_queryset_with_price_save_percent_annotation(self):
        queryset = Product.objects.with_price_save_percent_annotation()
        product1 = queryset.get(id=self.product1.id)
        product2 = queryset.get(id=self.product2.id)

        expected_discount_value1 = (
            self.product1.price.amount * self.product1.discount_percent
        ) / 100
        expected_price_save_percent1 = (
            expected_discount_value1 / self.product1.price.amount
        ) * 100

        self.assertEqual(
            product1.price_save_percent_annotation, expected_price_save_percent1
        )
        self.assertEqual(product2.price_save_percent_annotation, Decimal("0"))

    def test_queryset_with_likes_count_annotation(self):
        queryset = Product.objects.with_likes_count_annotation()
        product1 = queryset.get(id=self.product1.id)
        product2 = queryset.get(id=self.product2.id)

        self.assertEqual(product1.likes_count_annotation, 2)
        self.assertEqual(product2.likes_count_annotation, 0)

    def test_queryset_with_review_average_annotation(self):
        queryset = Product.objects.with_review_average_annotation()
        product1 = queryset.get(id=self.product1.id)
        product2 = queryset.get(id=self.product2.id)

        actual_avg = ProductReview.objects.filter(
            product=self.product1
        ).aggregate(avg=Avg("rate"))["avg"]

        self.assertEqual(product1.review_average_annotation, actual_avg)
        self.assertEqual(product2.review_average_annotation, 0)

    def test_annotations_match_property_values(self):
        """Test that model properties and annotations give consistent results"""
        queryset = (
            Product.objects.with_discount_value_annotation()
            .with_final_price_annotation()
            .with_price_save_percent_annotation()
            .with_likes_count_annotation()
            .with_review_average_annotation()
        )
        product1 = queryset.get(id=self.product1.id)

        product_instance = Product.objects.get(id=self.product1.id)

        self.assertEqual(
            product1.discount_value_annotation,
            product_instance.discount_value.amount,
        )
        self.assertEqual(
            product1.final_price_annotation, product_instance.final_price.amount
        )
        self.assertEqual(
            product1.price_save_percent_annotation,
            product_instance.price_save_percent,
        )
        self.assertEqual(
            product1.likes_count_annotation, product_instance.likes_count
        )
        self.assertEqual(
            product1.review_average_annotation, product_instance.review_average
        )

    def test_ordering_with_annotations(self):
        ProductFactory(
            price=Decimal("200.00"), discount_percent=Decimal("30.0")
        )
        ProductFactory(
            price=Decimal("300.00"), discount_percent=Decimal("10.0")
        )

        products_by_discount = list(
            Product.objects.with_discount_value_annotation().order_by(
                "-discount_value_annotation"
            )
        )
        self.assertTrue(
            products_by_discount[0].discount_value_annotation
            >= products_by_discount[1].discount_value_annotation
        )

        products_by_price = list(
            Product.objects.with_final_price_annotation().order_by(
                "-final_price_annotation"
            )
        )
        self.assertTrue(
            products_by_price[0].final_price_annotation
            >= products_by_price[1].final_price_annotation
        )

    def test_chained_annotations(self):
        """Test that queryset methods can be chained properly"""
        queryset = (
            Product.objects.with_likes_count_annotation()
            .with_review_average_annotation()
            .with_final_price_annotation()
        )

        product1 = queryset.get(id=self.product1.id)

        self.assertTrue(hasattr(product1, "likes_count_annotation"))
        self.assertTrue(hasattr(product1, "review_average_annotation"))
        self.assertTrue(hasattr(product1, "final_price_annotation"))

    def test_manager_methods(self):
        """Test that manager methods work correctly"""
        # Test likes count annotation
        likes_queryset = Product.objects.with_likes_count_annotation()
        self.assertTrue(likes_queryset.exists())

        # Test review average annotation
        review_queryset = Product.objects.with_review_average_annotation()
        self.assertTrue(review_queryset.exists())

        # Test final price annotation
        price_queryset = Product.objects.with_final_price_annotation()
        self.assertTrue(price_queryset.exists())

    def test_filtering_with_annotations(self):
        """Test that annotations work correctly with filtering"""
        # Create products with different like counts
        user3 = UserAccountFactory(num_addresses=0)
        ProductFavouriteFactory(
            product=self.product1, user=user3
        )  # Now has 3 likes

        queryset = Product.objects.with_likes_count_annotation().filter(
            likes_count_annotation__gte=2
        )

        self.assertEqual(queryset.count(), 1)
        self.assertEqual(queryset.first().id, self.product1.id)

    def test_empty_queryset_annotations(self):
        """Test annotations work on empty querysets"""
        Product.objects.all().delete()

        queryset = Product.objects.with_likes_count_annotation()
        self.assertEqual(queryset.count(), 0)

    def test_annotation_with_none_values(self):
        """Test annotations handle None values correctly"""
        product_no_vat = ProductFactory(vat=None, price=Decimal("100.00"))

        queryset = Product.objects.with_vat_value_annotation()
        product = queryset.get(id=product_no_vat.id)

        self.assertEqual(product.vat_value_annotation, Decimal("0.00"))



import logging
from typing import Any

from django.utils.translation import gettext_lazy as _
from djmoney.money import Money

from order.enum.status import OrderStatus, PaymentStatus
from order.models import Order
from order.payment import get_payment_provider
from pay_way.models import PayWay

logger = logging.getLogger(__name__)


class PayWayService:
    @staticmethod
    def get_provider_for_pay_way(pay_way: PayWay):
        if not pay_way.provider_code:
            logger.warning(f"PayWay {pay_way.id} has no provider_code defined")
            return None

        try:
            return get_payment_provider(pay_way.provider_code)
        except ValueError as e:
            logger.error(f"Failed to get payment provider: {e}")
            return None

    @staticmethod
    def process_payment(
        pay_way: PayWay, order: Order, **kwargs
    ) -> tuple[bool, dict[str, Any]]:
        if not pay_way.is_online_payment:
            payment_data = {
                "payment_id": f"OFFLINE_{order.id}",
                "status": PaymentStatus.PENDING,
                "amount": str(order.total_price.amount),
                "currency": order.total_price.currency,
                "provider": pay_way.provider_code or "offline",
            }

            if not pay_way.requires_confirmation:
                order.payment_method = (
                    pay_way.safe_translation_getter("name", any_language=True)
                    or ""
                )
                order.payment_status = PaymentStatus.PENDING
                order.payment_id = payment_data["payment_id"]
                order.save(
                    update_fields=[
                        "payment_method",
                        "payment_status",
                        "payment_id",
                    ]
                )

            return True, payment_data

        provider = PayWayService.get_provider_for_pay_way(pay_way)
        if not provider:
            return False, {"error": _("Payment provider not available")}

        success, payment_data = provider.process_payment(
            amount=order.total_price, order_id=str(order.id), **kwargs
        )

        if success:
            order.payment_method = (
                pay_way.safe_translation_getter("name", any_language=True) or ""
            )
            order.payment_status = payment_data.get(
                "status", PaymentStatus.PROCESSING
            )
            order.payment_id = payment_data.get("payment_id", "")
            order.save(
                update_fields=["payment_method", "payment_status", "payment_id"]
            )

            if order.payment_status == PaymentStatus.COMPLETED:
                order.mark_as_paid(
                    payment_id=payment_data.get("payment_id", ""),
                    payment_method=order.payment_method,
                )

        return success, payment_data

    @staticmethod
    def check_payment_status(
        pay_way: PayWay, order: Order
    ) -> tuple[PaymentStatus, dict[str, Any]]:
        if not order.payment_id:
            return PaymentStatus.PENDING, {
                "error": _("No payment ID found for order")
            }

        if not pay_way.is_online_payment:
            return order.payment_status, {
                "status": order.payment_status,
                "provider": "offline",
                "manual_check_required": pay_way.requires_confirmation,
            }

        provider = PayWayService.get_provider_for_pay_way(pay_way)
        if not provider:
            return PaymentStatus.PENDING, {
                "error": _("Payment provider not available")
            }

        status, status_data = provider.get_payment_status(order.payment_id)

        if status != order.payment_status:
            order.payment_status = status
            order.save(update_fields=["payment_status"])

            if status == PaymentStatus.COMPLETED:
                order.mark_as_paid(
                    payment_id=order.payment_id,
                    payment_method=order.payment_method,
                )

        return status, status_data

    @staticmethod
    def refund_payment(
        pay_way: PayWay, order: Order, amount: Money | None = None
    ) -> tuple[bool, dict[str, Any]]:
        if not order.payment_id:
            return False, {"error": _("No payment ID found for order")}

        if not pay_way.is_online_payment:
            if amount and amount.amount > 0:
                refund_info = {
                    "refund_id": f"MANUAL_REFUND_{order.id}",
                    "status": PaymentStatus.PENDING,
                    "amount": str(amount.amount),
                    "currency": amount.currency,
                    "provider": "manual",
                    "note": _("Manual refund process required"),
                }
            else:
                refund_info = {
                    "refund_id": f"MANUAL_REFUND_{order.id}",
                    "status": PaymentStatus.PENDING,
                    "amount": "full refund",
                    "provider": "manual",
                    "note": _("Manual refund process required"),
                }

            order.payment_status = PaymentStatus.REFUNDED
            order.status = OrderStatus.REFUNDED
            order.save(update_fields=["payment_status", "status"])

            return True, refund_info

        provider = PayWayService.get_provider_for_pay_way(pay_way)
        if not provider:
            return False, {"error": _("Payment provider not available")}

        success, refund_data = provider.refund_payment(order.payment_id, amount)

        if success:
            order.payment_status = PaymentStatus.REFUNDED
            order.status = OrderStatus.REFUNDED
            order.save(update_fields=["payment_status", "status"])

        return success, refund_data



from __future__ import annotations

from django.conf import settings
from django.db.models import Case, When, IntegerField, Q
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import (
    extend_schema,
    extend_schema_view,
    OpenApiParameter,
)
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response

from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.utils.serializers import (
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)
from core.utils.views import cache_methods
from product.filters.product import ProductFilter
from product.filters.review import ProductReviewFilter
from product.models.product import Product
from product.serializers.image import ProductImageSerializer
from product.serializers.product import (
    ProductDetailSerializer,
    ProductSerializer,
    ProductWriteSerializer,
)
from product.serializers.review import ProductReviewSerializer
from tag.filters.tag import TagFilter
from tag.serializers.tag import TagSerializer

req_serializers: RequestSerializersConfig = {
    "create": ProductWriteSerializer,
    "update": ProductWriteSerializer,
    "partial_update": ProductWriteSerializer,
}

res_serializers: ResponseSerializersConfig = {
    "create": ProductDetailSerializer,
    "list": ProductSerializer,
    "retrieve": ProductDetailSerializer,
    "update": ProductDetailSerializer,
    "partial_update": ProductDetailSerializer,
    "update_view_count": ProductDetailSerializer,
    "reviews": ProductReviewSerializer,
    "images": ProductImageSerializer,
    "tags": TagSerializer,
}

schema_config = create_schema_view_config(
    model_class=Product,
    display_config={
        "tag": "Products",
    },
    request_serializers=req_serializers,
    response_serializers=res_serializers,
)


@extend_schema_view(**schema_config)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class ProductViewSet(BaseModelViewSet):
    queryset = Product.objects.all()
    request_serializers = req_serializers
    response_serializers = res_serializers

    ordering_fields = [
        "price",
        "created_at",
        "active",
        "availability_priority",
        "view_count",
        "stock",
    ]
    ordering = ["-availability_priority", "id"]
    search_fields = ["translations__name", "translations__description", "slug"]

    def get_filterset_class(self):
        action_filter_map = {
            "reviews": ProductReviewFilter,
            "tags": TagFilter,
        }

        if self.action in action_filter_map:
            return action_filter_map[self.action]
        elif self.action == "images":
            return None
        return ProductFilter

    def get_queryset(self):
        queryset = super().get_queryset()
        queryset = queryset.annotate(
            availability_priority=Case(
                When(Q(active=True) & Q(stock__gt=0), then=1),
                default=0,
                output_field=IntegerField(),
            )
        )
        return queryset

    @property
    def filterset_class(self):
        return self.get_filterset_class()

    @extend_schema(
        operation_id="incrementProductViews",
        summary=_("Increment product view count"),
        description=_("Increment the view count for a product."),
        tags=["Products"],
        responses={
            200: ProductDetailSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["POST"],
    )
    def update_view_count(self, request, pk=None):
        product = self.get_object()
        product.view_count += 1
        product.save()

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            product, context=self.get_serializer_context()
        )
        return Response(response_serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listProductReviews",
        summary=_("Get product reviews"),
        description=_("Get all reviews for a product with pagination support."),
        tags=["Products"],
        parameters=[
            OpenApiParameter(
                name="id",
                type=int,
                location=OpenApiParameter.PATH,
                description="Product ID",
            ),
            OpenApiParameter(
                name="pagination_type",
                description=_("Pagination strategy type"),
                required=False,
                type=str,
                enum=["pageNumber", "cursor", "limitOffset"],
                default="pageNumber",
            ),
            OpenApiParameter(
                name="pagination",
                description=_("Enable or disable pagination"),
                required=False,
                type=str,
                enum=["true", "false"],
                default="true",
            ),
            OpenApiParameter(
                name="page_size",
                description=_("Number of results to return per page"),
                required=False,
                type=int,
                default=20,
            ),
        ],
        responses={
            200: {
                "type": "object",
                "properties": {
                    "links": {
                        "type": "object",
                        "properties": {
                            "next": {"type": "string", "nullable": True},
                            "previous": {"type": "string", "nullable": True},
                        },
                    },
                    "count": {"type": "integer"},
                    "total_pages": {"type": "integer"},
                    "page_size": {"type": "integer"},
                    "page_total_results": {"type": "integer"},
                    "results": {
                        "type": "array",
                        "items": {"$ref": "#/components/schemas/ProductReview"},
                    },
                },
            },
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def reviews(self, request, pk=None):
        product = get_object_or_404(Product, pk=pk)
        reviews = product.reviews.all()

        response_serializer_class = self.get_response_serializer()
        return self.paginate_and_serialize(
            reviews, request, serializer_class=response_serializer_class
        )

    @extend_schema(
        operation_id="listProductImages",
        summary=_("Get product images"),
        description=_("Get all images for a product."),
        tags=["Products"],
        parameters=[
            OpenApiParameter(
                name="id",
                type=int,
                location=OpenApiParameter.PATH,
                description="Product ID",
            ),
            OpenApiParameter(
                name="language",
                description=_("Language code for translations (el, en, de)"),
                required=False,
                type=str,
                enum=["el", "en", "de"],
                default="el",
                location=OpenApiParameter.QUERY,
            ),
        ],
        responses={
            200: {
                "type": "array",
                "items": {"$ref": "#/components/schemas/ProductImage"},
            },
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def images(self, request, pk=None):
        product = get_object_or_404(Product, pk=pk)
        images = product.images.all()

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            images, many=True, context=self.get_serializer_context()
        )
        return Response(response_serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="listProductTags",
        summary=_("Get product tags"),
        description=_("Get all tags associated with a product."),
        tags=["Products"],
        parameters=[
            OpenApiParameter(
                name="id",
                type=int,
                location=OpenApiParameter.PATH,
                description="Product ID",
            ),
        ],
        responses={
            200: {
                "type": "array",
                "items": {"$ref": "#/components/schemas/Tag"},
            },
            404: ErrorResponseSerializer,
        },
    )
    @action(
        detail=True,
        methods=["GET"],
    )
    def tags(self, request, pk=None):
        product = get_object_or_404(Product, pk=pk)
        tags = [tagged_item.tag for tagged_item in product.tags.all()]

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            tags, many=True, context=self.get_serializer_context()
        )
        return Response(response_serializer.data, status=status.HTTP_200_OK)



from django.conf import settings
from django.core.management.base import BaseCommand
from parler.utils.context import switch_language

from user.models.subscription import SubscriptionTopic


class Command(BaseCommand):
    help = "Set up default subscription topics for all configured languages"

    def handle(self, *args, **options):
        site_id = getattr(settings, "SITE_ID", 1)
        parler_languages = getattr(settings, "PARLER_LANGUAGES", {})

        languages = parler_languages.get(site_id, parler_languages.get(1, []))
        if not languages:
            self.stdout.write(
                self.style.ERROR(
                    "No languages found in PARLER_LANGUAGES setting"
                )
            )
            return

        language_codes = [lang["code"] for lang in languages]

        self.stdout.write(
            self.style.SUCCESS(
                f"Creating topics for languages: {', '.join(language_codes)}"
            )
        )

        topics_translations = {
            "en": [
                {
                    "slug": "weekly-newsletter",
                    "name": "Weekly Newsletter",
                    "description": "Get our weekly roundup of news, updates, and featured content",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": True,
                },
                {
                    "slug": "product-updates",
                    "name": "Product Updates",
                    "description": "Be the first to know about new features and improvements",
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "promotional-offers",
                    "name": "Promotional Offers",
                    "description": "Exclusive deals and special offers for subscribers",
                    "category": SubscriptionTopic.TopicCategory.MARKETING,
                    "is_default": False,
                },
                {
                    "slug": "system-notifications",
                    "name": "System Notifications",
                    "description": "Important system updates and maintenance notices",
                    "category": SubscriptionTopic.TopicCategory.SYSTEM,
                    "is_default": True,
                    "requires_confirmation": False,
                },
                {
                    "slug": "monthly-digest",
                    "name": "Monthly Digest",
                    "description": "Monthly summary of activity and highlights",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": False,
                },
                {
                    "slug": "new-features",
                    "name": "New Feature Announcements",
                    "description": "Get notified when we launch new features",
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "tips-and-tricks",
                    "name": "Tips & Tricks",
                    "description": "Learn how to get the most out of our platform",
                    "category": SubscriptionTopic.TopicCategory.OTHER,
                    "is_default": False,
                },
            ],
            "el": [
                {
                    "slug": "weekly-newsletter",
                    "name": "Εβδομαδιαίο Newsletter",
                    "description": "Λάβετε την εβδομαδιαία συλλογή νέων, ενημερώσεων και περιεχομένου",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": True,
                },
                {
                    "slug": "product-updates",
                    "name": "Ενημερώσεις Προϊόντων",
                    "description": "Μάθετε πρώτοι για νέες λειτουργίες και βελτιώσεις",  # noqa: RUF001
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "promotional-offers",
                    "name": "Προωθητικές Προσφορές",
                    "description": "Αποκλειστικές προσφορές για συνδρομητές",  # noqa: RUF001
                    "category": SubscriptionTopic.TopicCategory.MARKETING,
                    "is_default": False,
                },
                {
                    "slug": "system-notifications",
                    "name": "Ειδοποιήσεις Συστήματος",
                    "description": "Σημαντικές ενημερώσεις συστήματος και ειδοποιήσεις συντήρησης",
                    "category": SubscriptionTopic.TopicCategory.SYSTEM,
                    "is_default": True,
                    "requires_confirmation": False,
                },
                {
                    "slug": "monthly-digest",
                    "name": "Μηνιαία Περίληψη",
                    "description": "Μηνιαία περίληψη δραστηριότητας και σημαντικών στιγμών",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": False,
                },
                {
                    "slug": "new-features",
                    "name": "Ανακοινώσεις Νέων Λειτουργιών",
                    "description": "Ενημερωθείτε όταν κυκλοφορούν νέες λειτουργίες",
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "tips-and-tricks",
                    "name": "Συμβουλές & Κόλπα",
                    "description": "Μάθετε πώς να αξιοποιήσετε στο έπακρο την πλατφόρμα μας",  # noqa: RUF001
                    "category": SubscriptionTopic.TopicCategory.OTHER,
                    "is_default": False,
                },
            ],
            "de": [
                {
                    "slug": "weekly-newsletter",
                    "name": "Wöchentlicher Newsletter",
                    "description": "Erhalten Sie unsere wöchentliche Zusammenfassung von Nachrichten, Updates und Inhalten",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": True,
                },
                {
                    "slug": "product-updates",
                    "name": "Produkt-Updates",
                    "description": "Erfahren Sie als Erste von neuen Funktionen und Verbesserungen",
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "promotional-offers",
                    "name": "Werbeaktionen",
                    "description": "Exklusive Angebote und Sonderaktionen für Abonnenten",
                    "category": SubscriptionTopic.TopicCategory.MARKETING,
                    "is_default": False,
                },
                {
                    "slug": "system-notifications",
                    "name": "System-Benachrichtigungen",
                    "description": "Wichtige System-Updates und Wartungshinweise",
                    "category": SubscriptionTopic.TopicCategory.SYSTEM,
                    "is_default": True,
                    "requires_confirmation": False,
                },
                {
                    "slug": "monthly-digest",
                    "name": "Monatliche Zusammenfassung",
                    "description": "Monatliche Zusammenfassung von Aktivitäten und Highlights",
                    "category": SubscriptionTopic.TopicCategory.NEWSLETTER,
                    "is_default": False,
                },
                {
                    "slug": "new-features",
                    "name": "Neue Feature-Ankündigungen",
                    "description": "Benachrichtigung bei neuen Funktionen",
                    "category": SubscriptionTopic.TopicCategory.PRODUCT,
                    "is_default": False,
                },
                {
                    "slug": "tips-and-tricks",
                    "name": "Tipps & Tricks",
                    "description": "Lernen Sie, wie Sie das Beste aus unserer Plattform herausholen",
                    "category": SubscriptionTopic.TopicCategory.OTHER,
                    "is_default": False,
                },
            ],
        }

        created_count = 0
        updated_count = 0

        default_topics = topics_translations.get("en", [])

        for topic_data in default_topics:
            slug = topic_data["slug"]

            topic, created = SubscriptionTopic.objects.get_or_create(
                slug=slug,
                defaults={
                    "category": topic_data["category"],
                    "is_default": topic_data.get("is_default", False),
                    "requires_confirmation": topic_data.get(
                        "requires_confirmation", True
                    ),
                },
            )

            if created:
                created_count += 1
                self.stdout.write(self.style.SUCCESS(f"Created topic: {slug}"))
            else:
                updated_count += 1
                self.stdout.write(self.style.WARNING(f"Updated topic: {slug}"))

            for language_code in language_codes:
                if language_code in topics_translations:
                    lang_topics = topics_translations[language_code]
                    lang_topic_data = next(
                        (t for t in lang_topics if t["slug"] == slug), None
                    )

                    if lang_topic_data:
                        with switch_language(topic, language_code):
                            topic.name = lang_topic_data["name"]
                            topic.description = lang_topic_data["description"]
                            topic.save()

                        self.stdout.write(
                            self.style.SUCCESS(
                                f"  Added {language_code} translation for: {lang_topic_data['name']}"
                            )
                        )
                    else:
                        self.stdout.write(
                            self.style.WARNING(
                                f"  No {language_code} translation found for: {slug}"
                            )
                        )
                else:
                    self.stdout.write(
                        self.style.WARNING(
                            f"  No translations available for language: {language_code}"
                        )
                    )

        self.stdout.write(
            self.style.SUCCESS(
                f"\nSummary: {created_count} topics created, "
                f"{updated_count} topics updated for {len(language_codes)} languages."
            )
        )



from django.db import models
from django.utils.translation import gettext_lazy as _


class SeoModel(models.Model):
    seo_title = models.CharField(
        _("Seo Title"), max_length=70, blank=True, default=""
    )
    seo_description = models.TextField(
        _("Seo Description"), max_length=300, blank=True, default=""
    )
    seo_keywords = models.CharField(
        _("Seo Keywords"), max_length=255, blank=True, default=""
    )

    class Meta:
        abstract = True



# Generated by Django 5.0.6 on 2024-06-04 12:05
from django.db import migrations

import core.fields.image


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0014_alter_blogposttranslation_search_document_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogcategory",
            name="image",
            field=core.fields.image.ImageAndSvgField(
                blank=True,
                null=True,
                upload_to="uploads/blog/",
                verbose_name="Image",
            ),
        ),
    ]



from .author import BlogAuthorFactory
from .category import BlogCategoryFactory
from .comment import BlogCommentFactory
from .post import BlogPostFactory
from .tag import BlogTagFactory

__all__ = [
    "BlogAuthorFactory",
    "BlogCategoryFactory",
    "BlogCommentFactory",
    "BlogPostFactory",
    "BlogTagFactory",
]



from datetime import timedelta

from django.utils import timezone
from parler.managers import TranslatableManager, TranslatableQuerySet


class OrderHistoryQuerySet(TranslatableQuerySet):
    def for_order(self, order):
        return self.filter(order=order)

    def by_change_type(self, change_type):
        return self.filter(change_type=change_type)

    def by_user(self, user):
        return self.filter(user=user)

    def recent(self, days=7):
        cutoff_date = timezone.now() - timedelta(days=days)
        return self.filter(created_at__gte=cutoff_date)

    def status_changes(self):
        return self.filter(change_type="STATUS")

    def payment_changes(self):
        return self.filter(change_type="PAYMENT")

    def system_changes(self):
        return self.filter(user__isnull=True)

    def user_changes(self):
        return self.filter(user__isnull=False)


class OrderHistoryManager(TranslatableManager):
    def get_queryset(self) -> OrderHistoryQuerySet:
        return OrderHistoryQuerySet(self.model, using=self._db)

    def for_order(self, order):
        return self.get_queryset().for_order(order)

    def by_change_type(self, change_type):
        return self.get_queryset().by_change_type(change_type)

    def by_user(self, user):
        return self.get_queryset().by_user(user)

    def recent(self, days=7):
        return self.get_queryset().recent(days)

    def status_changes(self):
        return self.get_queryset().status_changes()

    def payment_changes(self):
        return self.get_queryset().payment_changes()

    def system_changes(self):
        return self.get_queryset().system_changes()

    def user_changes(self):
        return self.get_queryset().user_changes()


class OrderItemHistoryQuerySet(TranslatableQuerySet):
    def for_order_item(self, order_item):
        return self.filter(order_item=order_item)

    def for_order(self, order):
        return self.filter(order_item__order=order)

    def by_change_type(self, change_type):
        return self.filter(change_type=change_type)

    def quantity_changes(self):
        return self.filter(change_type="QUANTITY")

    def price_changes(self):
        return self.filter(change_type="PRICE")

    def refunds(self):
        return self.filter(change_type="REFUND")


class OrderItemHistoryManager(TranslatableManager):
    def get_queryset(self) -> OrderItemHistoryQuerySet:
        return OrderItemHistoryQuerySet(self.model, using=self._db)

    def for_order_item(self, order_item):
        return self.get_queryset().for_order_item(order_item)

    def for_order(self, order):
        return self.get_queryset().for_order(order)

    def by_change_type(self, change_type):
        return self.get_queryset().by_change_type(change_type)

    def quantity_changes(self):
        return self.get_queryset().quantity_changes()

    def price_changes(self):
        return self.get_queryset().price_changes()

    def refunds(self):
        return self.get_queryset().refunds()



import glob
import os

# Set the path to the project root directory
root_dir = "../."

# Define a list of folders to exclude
excluded_folders = ["core"]  # Add the folders you want to exclude

# Find all the Django apps in the project
app_dirs = glob.glob(os.path.join(root_dir, "*/migrations"))

# Iterate over the app directories
for app_dir in app_dirs:
    # Get the app name from the directory path
    app_name = os.path.basename(os.path.dirname(app_dir))

    # Check if the app is in the excluded list
    if app_name in excluded_folders:
        print(f"Ignoring migrations in {app_name}")
        continue

    # Find all files in 'migrations' folder and delete all except __init__.py
    migration_files = glob.glob(os.path.join(app_dir, "*"))
    print(f"len(migration_files) in {app_name}: {len(migration_files)}")
    if len(migration_files) == 0:
        print(f"No migration files found in {app_name}")
        pass

    for migration_file in migration_files:
        if migration_file.endswith("__init__.py") or migration_file.endswith(
            "__pycache__"
        ):
            continue
        os.remove(migration_file)
        print(f"Deleted: {migration_file} in {app_name}")



import os

from django.contrib.postgres.indexes import BTreeIndex
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from parler.fields import TranslationsForeignKey
from parler.models import TranslatableModel, TranslatedFieldsModel
from tinymce.models import HTMLField

from blog.managers.post import BlogPostManager
from core.fields.image import ImageAndSvgField
from core.models import PublishableModel, TimeStampMixinModel, UUIDModel
from core.utils.generators import SlugifyConfig, unique_slugify
from meili.models import IndexMixin
from seo.models import SeoModel


class BlogPost(
    TranslatableModel,
    SeoModel,
    TimeStampMixinModel,
    PublishableModel,
    UUIDModel,
):
    id = models.BigAutoField(primary_key=True)
    slug = models.SlugField(_("Slug"), max_length=255, unique=True)
    image = ImageAndSvgField(
        _("Image"), upload_to="uploads/blog/", blank=True, null=True
    )
    likes = models.ManyToManyField(
        "user.UserAccount", related_name="liked_blog_posts", blank=True
    )
    category = models.ForeignKey(
        "blog.BlogCategory",
        related_name="blog_posts",
        on_delete=models.SET_NULL,
        null=True,
    )
    tags = models.ManyToManyField(
        "blog.BlogTag", related_name="blog_posts", blank=True
    )
    author = models.ForeignKey(
        "blog.BlogAuthor",
        related_name="blog_posts",
        on_delete=models.SET_NULL,
        null=True,
    )
    featured = models.BooleanField(_("Featured"), default=False)
    view_count = models.PositiveBigIntegerField(_("View Count"), default=0)

    objects: BlogPostManager = BlogPostManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Blog Post")
        verbose_name_plural = _("Blog Posts")
        ordering = ["-published_at"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            *PublishableModel.Meta.indexes,
            BTreeIndex(fields=["view_count"], name="blog_post_view_count_ix"),
            BTreeIndex(fields=["featured"], name="blog_post_featured_ix"),
            BTreeIndex(fields=["category"], name="blog_post_category_ix"),
            BTreeIndex(fields=["author"], name="blog_post_author_ix"),
            BTreeIndex(fields=["slug"], name="blog_post_slug_ix"),
        ]

    def __str__(self):
        title = (
            self.safe_translation_getter("title", any_language=True)
            or "Untitled"
        )
        author_name = self.author.user.email if self.author else "Unknown"
        return f"{title} by {author_name}"

    def save(self, *args, **kwargs):
        if not self.slug:
            config = SlugifyConfig(
                instance=self,
            )
            self.slug = unique_slugify(config)
        super().save(*args, **kwargs)

    @property
    def main_image_path(self) -> str:
        if self.image and hasattr(self.image, "name"):
            return f"media/uploads/blog/{os.path.basename(self.image.name)}"
        return ""

    @property
    def likes_count(self) -> int:
        return self.likes.count()

    @property
    def comments_count(self) -> int:
        return self.comments.filter(approved=True).count()

    @property
    def all_comments_count(self) -> int:
        return self.comments.count()

    @property
    def tags_count(self) -> int:
        return self.tags.filter(active=True).count()


class BlogPostTranslation(TranslatedFieldsModel, IndexMixin):
    master = TranslationsForeignKey(
        "blog.BlogPost",
        on_delete=models.CASCADE,
        related_name="translations",
        null=True,
    )
    title = models.CharField(_("Title"), max_length=255, blank=True, default="")
    subtitle = models.CharField(
        _("Subtitle"), max_length=255, blank=True, default=""
    )
    body = HTMLField(_("Body"), blank=True, null=True)

    class Meta:
        app_label = "blog"
        db_table = "blog_blogpost_translation"
        unique_together = ("language_code", "master")
        verbose_name = _("Blog Post Translation")
        verbose_name_plural = _("Blog Post Translations")

    class MeiliMeta:
        filterable_fields = ("title", "language_code", "likes_count")
        searchable_fields = ("id", "title", "subtitle", "body")
        displayed_fields = ("id", "title", "subtitle", "body", "language_code")
        sortable_fields = ("likes_count",)
        ranking_rules = [
            "words",
            "typo",
            "proximity",
            "attribute",
            "sort",
            "likes_count:desc",
            "exactness",
        ]
        synonyms = {
            "blog": ["article", "post"],
            "article": ["blog", "post"],
            "post": ["blog", "article"],
            "tutorial": ["guide", "how-to"],
            "guide": ["tutorial", "how-to"],
            "how-to": ["tutorial", "guide"],
            "υπερθέρμανση": ["καίει", "καίγεται"],
            "καίει": ["καίγεται", "υπερθέρμανση"],
            "καίγεται": ["καίει", "υπερθέρμανση"],
        }
        typo_tolerance = {
            "enabled": True,
            "minWordSizeForTypos": {"oneTypo": 4, "twoTypos": 8},
            "disableOnWords": ["specific"],
            "disableOnAttributes": ["id"],
        }
        faceting = {"maxValuesPerFacet": 50}
        pagination = {"maxTotalHits": 1000}

    @classmethod
    def get_additional_meili_fields(cls):
        return {"likes_count": lambda obj: obj.master.likes_count}

    def __str__(self):
        title = self.title or "Untitled"
        return f"{title} ({self.language_code})"



# Generated by Django 5.0.4 on 2024-04-17 15:43
from decimal import Decimal

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0011_alter_productcategory_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="product",
            name="discount_percent",
            field=models.DecimalField(
                decimal_places=2,
                default=Decimal("0"),
                max_digits=11,
                verbose_name="Discount Percent",
            ),
        ),
    ]
